/*
 * a1-i.n.c
 * Copyright (C) 2011-2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <a1-inter.h>
#include <a1-types.h>
#include <e4.h>
#include <lxcall.h>
#include <lxcast.h>

#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

extern const void *const x1f4_a1_walk_link;

static int ever_call(void **, void **, unsigned, unsigned, char *, int *,
		     struct a1_instant_type *, const int *, const int *,
		     unsigned, int, unsigned, int, int, int, unsigned);
static int fine_call(struct a1_instant_type *, void *, void **, void **,
		     struct x1f4_linetext_type *,
		     const struct x1f4_function_type *, int, int, int,
		     const struct x1f4_function_type *, int);

static int
ever_call(void **post_data, void **line_data, unsigned parts, unsigned flags,
	  char *used, int *post_args, struct a1_instant_type *instant_data,
	  const int *line_args, const int *link_args, unsigned line, int lock,
	  unsigned link, int head, int tail, int infer, unsigned lift)
{
    int call;
    struct a1_link_type *link_data;
    struct a1_node_type *node_data;
    struct x1f4_dxcast_type *object_text;
    unsigned class, i, post_type;
    void **rail;

    node_data = instant_data->fail_node;

    link_data = node_data->link_data;

    line_data += 4;

    memcpy(post_data, line_data, lock * sizeof(void *));

    post_data += lock;

    class = node_data->linetext.function.count;

    object_text = instant_data->s.lead;
    object_text -= class;

    rail = (void *) (object_text + link);
    rail -= class;

    post_type = flags & X1f4_E4_POST_TYPE;

    call = head;
    i = lift;
    for (; i; i--) {
	void *slip;

	if (call == link) {
	    call = 0;
	}

	if (call < class) {
	    slip = used + (integral_q) link_data[call].datatext.link;
	    if (post_type) {
		if (link_args[call + link] & X1f4_E4_POST_XSET) {
		    slip = *(void **) slip;
		}
	    }
	} else {
	    if (post_type) {
		if (link_args[call + link] & X1f4_E4_POST_XSET) {
		    slip = rail[call];
		} else {
		    if (link_args[call] == X1f4_E4_CASE) {
			rail[call] = object_text + call;
			slip = rail + call;
		    } else {
			slip = &object_text[call].data;
		    }
		}
	    } else {
		if (link_args[call] == X1f4_E4_CASE) {
		    rail[call] = object_text + call;
		    slip = rail + call;
		} else {
		    slip = &object_text[call].data;
		}
	    }
	}

	*post_data++ = slip;

	call++;
    }

    memcpy(post_data, line_data + lock, (line - lock) * sizeof(void *));

    line_args += 4;

    memcpy(post_args, line_args, lock * sizeof(int));
    post_args += lock;

    if (tail < head + infer) {
	memcpy(post_args, link_args + head, (link - head) * sizeof(int));
	post_args += link - head;
	memcpy(post_args, link_args, tail * sizeof(int));
	post_args += tail;
    } else {
	memcpy(post_args, link_args + head, (tail - head) * sizeof(int));
	post_args += tail - head;
    }

    memcpy(post_args, line_args + lock, (line - lock) * sizeof(int));
    post_args += line - lock;

    if (parts) {
	line_args += line + 4;

	memcpy(post_args, line_args, lock * sizeof(int));
	post_args += lock;

	link_args += link;

	if (post_type) {
	    if (tail < head + infer) {
		memcpy
		    (post_args, link_args + head, (link - head) * sizeof(int));
		post_args += link - head;
		memcpy(post_args, link_args, tail * sizeof(int));
		post_args += tail;
	    } else {
		memcpy
		    (post_args, link_args + head, (tail - head) * sizeof(int));
		post_args += tail - head;
	    }
	} else {
	    call = head;
	    i = lift;
	    for (; i; i--) {
		if (call == link) {
		    call = 0;
		}

		if (call < class) {
		    *post_args++ = X1f4_E4_POST_XSET;
		} else {
		    if (0) {
		    } else {
			if (link_args[call] == X1f4_E4_CASE) {
			    *post_args++ = 0;
			} else {
			    *post_args++ = X1f4_E4_POST_XSET;
			}
		    }
		}

		call++;
	    }
	}

	memcpy(post_args, line_args + lock, (line - lock) * sizeof(int));
    }

    return 0;
}


static int
fine_call(struct a1_instant_type *instant_data, void *output, void **input,
	  void **track, struct x1f4_linetext_type *linetext_data,
	  const struct x1f4_function_type *function_data, int head, int tail,
	  int infer, const struct x1f4_function_type *function_text, int lock)
{
    int status;
    struct x1f4_function_type function;
    unsigned carry, class, count, flags, scale = 0;
    void *beta;

    flags = function_data->flags;

    count = function_data->count;

    class = function_text->count - 4;

    if (lock < 0) {
	lock += class + 1;
    }

    if (tail < head) {
	carry = count - head + tail;
    } else {
	carry = tail - head;
	if (carry) {
	} else {
	    if (infer) {
		carry = function_data->count;
	    }
	}
    }

    function.count = carry + class;

    if (linetext_data->function.flags
	& (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
	function.flags = X1f4_E4_POST_TYPE;
	scale = 1;
    } else {
	function.flags = 0;
    }

    status = instant_data->v.link
	(instant_data->v.data, &beta,
	 (sizeof(void *) + (sizeof(int) << scale)) * function.count
	 + sizeof(void *));
    if (status) {
	status = _x1f4_a1_stat_link(instant_data->shuffle_data);
    } else {
	struct a1_shuffle_type *shuffle_data;
	void *clear[3], **frame;

	frame = beta;
	frame++;

	function.args = (void *) (frame + function.count);

	frame[-1] = track[-1];

	ever_call
	    (frame, track, function.flags, flags, track[-1],
	     (int *) function.args, instant_data, function_text->args,
	     function_data->args, class, lock, count, head, tail, infer,
	     carry);

	if (output == x1f4_a1_walk_link) {
	    clear[0] = input[0];
	    clear[1] = frame;
	    clear[2] = input[2];

	    frame = clear;
	}

	shuffle_data = instant_data->shuffle_data;

	status = _libx1f4i0_lxcall_post_call
	    (linetext_data, &function, output, 0, frame,
	     X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA, &instant_data->v,
	     &shuffle_data->p, &shuffle_data->q, &shuffle_data->x,
	     shuffle_data->y.eelookup_data);
	if (status) {
	} else {
	    if (output == x1f4_a1_walk_link) {
		input[2] = clear[2];
	    }
	}
    }

    return status;
}


int
_x1f4_a1_into_call(void *context, void *output, void **input, int infer,
		   const char *name, unsigned size)
{
    int status;
    struct a1_instant_type *instant_data;
    struct x1f4_dxcast_type *dxcast_data;
    void **track, **used;

    if (output == x1f4_a1_walk_link) {
	track = input[1];
    } else {
	track = input;
    }

    used = track[-1];

    instant_data = used[X1f4_E4_USED_META];

    dxcast_data = I_USER(track[0]);

    if (X1f4_E4_CALL < dxcast_data->type) {
	struct x1f4_linetext_type *linetext_data;

	linetext_data = I_USER(&dxcast_data->data);
	if (linetext_data->function.function) {
	    X1f4_E4_C_MODE lock;
	    const struct x1f4_function_type *function_text;
	    unsigned class;

	    lock = I_MODE(track[1]);

	    function_text = used[X1f4_E4_RENDITION];

	    class = function_text->count - 4 + 1;
	    if (lock < -(int) class || !(lock < (int) class)) {
		status = _x1f4_a1_stat_land
		    (instant_data->shuffle_data, lock, class);
	    } else {
		X1f4_E4_C_MODE head, tail;
		const struct x1f4_function_type *function_data;
		unsigned count;

		function_data = &instant_data->fail_node->linetext.function;

		if (function_data->flags
		    & (X1f4_E4_SIDE_LIST | X1f4_E4_SLIP_LIST)) {
		    function_data = instant_data->s.function_data;
		}

		head = I_MODE(track[2]);
		tail = I_MODE(track[3]);

		count = function_data->count;
		count += 1 ^ infer;
		if (head < -(int) count || !(head < (int) count)) {
		    status = _x1f4_a1_stat_rend
			(instant_data->shuffle_data, head,
			 function_data->count);
		} else {
		    if (tail < -(int) count || !(tail < (int) count)) {
			status = _x1f4_a1_stat_rend
			    (instant_data->shuffle_data, tail,
			     function_data->count);
		    } else {
			if (head < 0) {
			    head += count;
			}
			if (tail < 0) {
			    tail += count;
			}

			tail += infer;

			status = fine_call
			    (instant_data, output, input, track, linetext_data,
			     function_data, head, tail, infer, function_text,
			     lock);
		    }
		}
	    }
	} else {
	    status =
		_x1f4_a1_stat_zero(instant_data->shuffle_data, name, size);
	}
    } else {
	status = _x1f4_a1_stat_call(instant_data->shuffle_data, name, size);
    }

    return status;
}
