/*
 * file.d.h.c
 * Copyright (C) 2008-2010, 2013, 2014, 2016, 2018, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>

#include <file-defs.h>
#include <file-types.h>
#include <trans.h>

#define data(file, offset) \
    ((char *) (file(file) + 1) + (offset))

#define file(file) \
    ((struct eifile_type *) (file))

int
_libx1f4i0_read_ifile(void *file, void **line, unsigned *size, unsigned call,
		      int fill, struct trans_type *trans_data)
{
    int status;

    do {
	int done = OVER_CLASS, (*pull)(void *, void *, unsigned);
	unsigned fast, hold, miss;
	void *data, *into, *sd;

	data = *line;

	miss = file(file)->size;
	if (miss) {
	    unsigned ever;

	    ever = file(file)->ever;

	    if (miss < call) {
		memcpy(data, data(file, ever), miss);
		if (EVER_MAX < call) {
		    hold = call - miss;
		    into = (char *) data + miss;
		} else {
		    hold = EVER_MAX;
		    into = file(file) + 1;
		}
	    } else {
		char *text;

		memcpy(data, data(file, ever), call);
		text = data;
		text[call] = 0;

		*size = call;

		file(file)->ever = ever + call;
		file(file)->size = miss - call;

		status = 0;

		break;
	    }
	} else {
	    if (EVER_MAX < call) {
		hold = call;
		into = data;
	    } else {
		hold = EVER_MAX;
		into = file(file) + 1;
	    }
	}

	fast = miss;

	pull = file(file)->sf->pull;

	sd = &file(file)->sd;

	while (1) {
	    status = pull(sd, into, hold);
	    if (status == -1) {
		if (errno == EINTR) {
		    continue;
		} else {
		    if (errno == EAGAIN || errno == EWOULDBLOCK) {
			done = NONE_CLASS;

			status = 0;
		    } else {
			status = READ_CLASS;

			if (1) {
			    break;
			}
		    }
		}
	    }

	    if (status) {
		unsigned ever;

		ever = status;

		fast += ever;
		if (fast < call) {
		    hold -= ever;
		    into = (char *) into + ever;
		} else {
		    char *text;

		    if (EVER_MAX < call) {
			file(file)->size = 0;
		    } else {
			unsigned slip;

			slip = call - miss;

			memcpy((char *) data + miss, file(file) + 1, slip);

			file(file)->ever = slip;
			file(file)->size = fast - call;

			fast = call;
		    }

		    text = data;

		    *size = fast;

		    text[fast] = 0;

		    status = 0;

		    break;
		}
	    } else {
		if (fast && !fill) {
		    char *text;

		    file(file)->size = 0;

		    if (EVER_MAX < call) {
		    } else {
			memcpy((char *) data + miss, file(file) + 1,
			       fast - miss);
		    }

		    text = data;

		    if (fast ^ call) {
			status = trans_data->mode
			    (trans_data->data, &data, fast + 1);
			if (status) {
			    status = MODE_CLASS;

			    break;
			} else {
			    text = data;
			}
		    }

		    *line = data;
		    *size = fast;

		    text[fast] = 0;
		} else {
		    file(file)->size = 0;

		    status = 0;
		    if (status) {
			status = FREE_CLASS;
		    } else {
			status = done;

			if (fast) {
			    file(file)->sf->seek(sd, -(int) fast, SEEK_CUR);
			}
		    }
		}

		break;
	    }
	}
    } while (0);

    return status;
}
