/*
 * fine.q.c
 * Copyright (C) 2002-2012, 2018, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <lxvery-config.h>

#include <c-types.h>
#include <integral_q.h>

#define __BINARY_LOOK__			0
#define __LINEAR_LOOK__			1

#define __BINARY_PICK__			0
#define __LINEAR_PICK__			1
#define __UNROLL_PICK__			0

#if SIZEOF_VOID_P == 8
# define HALF				040
#else
# define HALF				020
#endif				/* SIZEOF_VOID_P == 8 */

#define VX				65536

#if __VERY_PSET__
# define F_TYPE				C_BILL
#else
# define F_TYPE				C_MODE
#endif				/* __VERY_PSET__ */

int
libx1f4i0_fine_fare(void *text, void *node)
{
    return *(F_TYPE *) text < *(F_TYPE *) node
	? -1 : *(F_TYPE *) text ^ *(F_TYPE *) node ? 1 : 0;
}


int
libx1f4i0_fine_look(void *text, void *node, unsigned *pick)
{
    int status;
    unsigned size;

    size = *(integral_q *) node;
    if (size) {
	F_TYPE *call, fine, *high;

	fine = *(F_TYPE *) text;

	high = node;

	call = high + 1;

#if __LINEAR_LOOK__
	for (--size; size; size--) {
	    if (fine <= *call) {
		break;
	    } else {
		call++;
	    }
	}
#else
	size--;
	while (size) {
	    F_TYPE *slip;
	    unsigned half;

	    half = (size + 1) >> 1;
	    slip = call + half;
	    if (fine < *slip) {
		size = half - 1;
	    } else {
		call = slip;
		size -= half;
	    }
	}
#endif				/* __LINEAR_LOOK__ */

	if (*call ^ fine) {
	    status = 0;

	    if (fine < *call) {
		call--;
	    } else {
	    }

	    *pick = call - high;
	} else {
	    status = 1;

	    *pick = call - high;
	}
    } else {
	status = 0;

	*pick = 0;
    }

    return status;
}


int
libx1f4i0_fine_move(void *target, void *source, unsigned count)
{
#if 1
    F_TYPE *s, *t;

    s = source;
    t = target;
    if (count ^ 1) {
	if (t < s) {
	    for (; count; count--) {
		*t++ = *s++;
	    }
	} else {
	    s += count;
	    if (t < s) {
		t += count;
		for (; count; count--) {
		    *--t = *--s;
		}
	    } else {
		s -= count;
		for (; count; count--) {
		    *t++ = *s++;
		}
	    }
	}
    } else {
	*t++ = *s++;
    }
#else
    memmove(target, source, count * sizeof(F_TYPE));
#endif				/* 0 */

    return 0;
}


int
libx1f4i0_fine_pick(void *text, void *node, unsigned *pick)
{
    F_TYPE fine, *high;
    int status;
#if __BINARY_PICK__
    unsigned half = HALF << 1;
#endif				/* __BINARY_PICK__ */
    unsigned slip;

    fine = *(F_TYPE *) text;

    high = node;

#if __UNROLL_PICK__
    slip = 0;

# if SIZEOF_VOID_P == 8
    if (fine < high[slip + 0040]) {
    } else {
	slip += 0040;
    }
# endif				/* SIZEOF_VOID_P == 8 */
    if (fine < high[slip + 0020]) {
    } else {
	slip += 0020;
    }
    if (fine < high[slip + 0010]) {
    } else {
	slip += 0010;
    }
    if (fine < high[slip + 0004]) {
    } else {
	slip += 0004;
    }
    if (fine < high[slip + 0002]) {
    } else {
	slip += 0002;
    }
#elif __LINEAR_PICK__
    high += 2;
    slip = (SIZEOF_VOID_P << 2) - 1;
    for (; slip; slip--) {
	if (fine < *high) {
	    break;
	} else {
	    high += 2;
	}
    }

    high = node;

    slip = ((SIZEOF_VOID_P << 2) - 1 - slip) << 1;
#else
    slip = 0;

    do {
	if (fine < high[slip + half]) {
	} else {
	    slip += half;
	}

	half >>= 1;
    } while (half ^ 1);
#endif				/* __UNROLL_PICK__ */

    high += slip;

    if (*(integral_q *) node & (integral_q) 2 << slip) {
	if (fine < high[1]) {
	    *pick = slip + 0;
	    if (slip) {
		status = fine == high[0];
	    } else {
		status = 0;
	    }
	} else {
	    *pick = slip + 1;
	    status = fine == high[1];
	}
    } else {
	*pick = slip + 1;
	if (slip) {
	    status = fine == high[0];
	} else {
	    if (1) {
		status = 0;
	    }
	}
    }

    return status;
}
