/*
 * lxcase-i.f.c
 * Copyright (C) 2016, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <e4.h>
#include <lxcall.h>
#include <lxcase-defs.h>
#include <lxcase-drive.h>
#include <lxcase-inter.h>
#include <lxcase-types.h>
#include <lxdate.h>

static int cast_date(struct lxcase_type *, time_t, void *);
static int cast_rate(struct lxcase_type *, X1f4_E4_C_MODE, void *);

static int
cast_date(struct lxcase_type *lxcase_data, time_t time, void *output)
{
    int status;
    struct x1f4_datatype_type *datatype_data;

    datatype_data = lxcase_data->link_l.date;
    if (datatype_data) {
	void *dxdate;

	status = datatype_data->line(datatype_data->context, &dxdate);
	if (status) {
	} else {
	    x1f4_sink_lxdate(dxdate, time, 0);

	    status = x1f4_hang_node
		(&lxcase_data->link_v, &lxcase_data->link_e, datatype_data,
		 datatype_data->type, dxdate, output);
	}
    } else {
	status = 1;
    }

    return status;
}


static int
cast_rate(struct lxcase_type *lxcase_data, X1f4_E4_C_MODE mode, void *output)
{
    return x1f4_cast_rate
	(&lxcase_data->link_v, &lxcase_data->link_e, X1f4_E4_MODE, &mode,
	 output);
}


int
_libx1f4i0_lxcase_half_file(void *context, void *output, void **input,
			    int (*stat)(void *, void *, void *))
{
    int status = 0;

    do {
	X1f4_E4_C_MODE mode = 0;
	const int *args;
	int date, type;
	struct stat info;
	struct x1f4_function_type *function_data;
	unsigned i, size;

	function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

	date = ((struct lxcase_type *) context)->link_i.date;

	size = function_data->count;

	args = function_data->args;

	i = 1;
	for (; i < size; i++) {
	    type = *++args;
	    if (i & 1) {
		if (type != X1f4_E4_MODE) {
		    status = _libx1f4i0_lxcase_stat_iran(context, i);
		    if (1) {
			break;
		    }
		}

		mode = I_MODE(input[i]);
		if (mode < ST_ATIME || ST_UID < mode) {
		    status = _libx1f4i0_lxcase_stat_park(context, mode);
		    if (1) {
			break;
		    }
		}
	    } else {
		if (type != X1f4_E4_MODE && type != date) {
		    status = _libx1f4i0_lxcase_stat_dori(context, i);
		    if (1) {
			break;
		    }
		} else {
		    if (type
			!= (mode == ST_ATIME || mode == ST_CTIME
			    || mode == ST_MTIME ? date : X1f4_E4_MODE)) {
			status = _libx1f4i0_lxcase_stat_mill(context, i);
			if (1) {
			    break;
			}
		    }
		}
	    }
	}
	if (i ^ size) {
	    break;
	}

	status = stat(context, *input, &info);
	if (status) {
	    break;
	}

	if (size < 2) {
	    status = cast_rate(context, 0, output);
	    if (1) {
		break;
	    }
	}

	args = function_data->args;

	i = 1;
	for (; i < size; i++) {
	    type = *++args;
	    input++;
	    if (i & 1) {
		mode = I_MODE(*input);
	    } else {
		switch (mode) {
		case ST_ATIME:
		    x1f4_sink_lxdate(I_USER(*input), info.st_atime, 0);
		    break;
		case ST_CTIME:
		    x1f4_sink_lxdate(I_USER(*input), info.st_ctime, 0);
		    break;
		case ST_GID:
		    l_MODE(info.st_gid, *input);
		    break;
		case ST_MODE:
		    l_MODE(info.st_mode, *input);
		    break;
		case ST_MTIME:
		    x1f4_sink_lxdate(I_USER(*input), info.st_mtime, 0);
		    break;
		case ST_NLINK:
		    l_MODE(info.st_nlink, *input);
		    break;
		case ST_SIZE:
		    l_MODE(info.st_size, *input);
		    break;
		case ST_UID:
		    l_MODE(info.st_uid, *input);
		    break;
		}
	    }
	}

	if (1) {
	    if (1) {
		switch (mode) {
		case ST_ATIME:
		    status = cast_date(context, info.st_atime, output);
		    break;
		case ST_CTIME:
		    status = cast_date(context, info.st_ctime, output);
		    break;
		case ST_GID:
		    status = cast_rate(context, info.st_gid, output);
		    break;
		case ST_MODE:
		    status = cast_rate(context, info.st_mode, output);
		    break;
		case ST_MTIME:
		    status = cast_date(context, info.st_mtime, output);
		    break;
		case ST_NLINK:
		    status = cast_rate(context, info.st_nlink, output);
		    break;
		case ST_SIZE:
		    status = cast_rate(context, info.st_size, output);
		    break;
		case ST_UID:
		    status = cast_rate(context, info.st_uid, output);
		    break;
		}
	    }
	}
    } while (0);

    return status;
}
