/*
 * lxdash-e.9.c
 * Copyright (C) 2008-2013, 2015, 2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <d_fset.h>
#include <exerrors.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>
#include <tcvset.h>

#define tccase(vset) \
    ((struct x1f4_tccase_type *) (vset))

int
x1f4_post_lxdash(void *subtext, void **address,
		 const struct excase_type *excase_data)
{
    int status;
    struct lxdash_type *lxdash_data;
    unsigned call;
    void *dxdash;

    lxdash_data = subtext;

    _libx1f4i0_call_lxfset(&call);

    status = lxdash_data->link_m.link
	(lxdash_data->link_m.data, &dxdash,
	 __struct_dxdash_type_alloc_size__ + call);
    if (status) {
	status = _libx1f4i0_lxdash_stat_link(lxdash_data);
    } else {
	dxdash = pick_dxdash(dxdash);

	status = _libx1f4i0_lxdash_fast_line(subtext, dxdash);
	if (status) {
	    lxdash_data->link_m.free
		(lxdash_data->link_m.data, lock_cxline(dxdash));
	} else {
	    void *pset;
	    const void *text;

	    ((struct dxdash_type *) dxdash)->size = 0;

	    lock_excase(dxdash)->nodetype_data = lxdash_data->link_s.nodetype;

	    lock_excase(dxdash)->slip = NULL;

	    lock_exline(dxdash)->call = 1;

	    _libx1f4i0_init_tcvset(&pset, &text);

	    status = tccase(text)->push
		(&pset, &text, excase_data, (void *) &lxdash_data->link_m);
	    if (status) {
		status = X1f4_EX_CRITICAL;

		_libx1f4i0_lxdash_slip_line(subtext, &dxdash);
	    } else {
		*address = dxdash;

		lock_exline(dxdash)->pset = pset;
		lock_exline(dxdash)->text = text;
	    }
	}
    }

    return status;
}
