/*
 * lxdash-i.i.c
 * Copyright (C) 2011-2018, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <d_fset.h>
#include <fd.h>
#include <it.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxdash-drive.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>

#define dxdash(list) \
    ((struct dxdash_type *) (list))
#define lxdash(list) \
    ((struct lxdash_type *) (list))

typedef struct rule_type {
    int roll;
    struct direct_type *direct_data;
    struct excase_type *excase_data;
    struct lxtext_type *lxtext_data;
} rule_type;

static int rule_dash(void *, void *);

static int
rule_dash(void *line, void *data)
{
    int link, status;
    struct direct_type *direct_data;
    struct effect_type *effect_data;
    struct rule_type *rule_data;

    rule_data = line;

    direct_data = rule_data->direct_data;

    effect_data = data;

    direct_data->link = effect_data->link;
    direct_data->type = effect_data->type;

    direct_data->fxdata = effect_data->fxdata;

    link = direct_data->link;
    if (~link) {
	struct lxtext_type *lxtext_data;

	lxtext_data = rule_data->lxtext_data + link;

	if (lxtext_data->lxtype.flags & X1f4_LX_LINK_ACCESS) {
	    status = lxtext_data->lxtype.node
		(lxtext_data->lxtype.context, (void *) &direct_data->fxdata,
		 I_USER(&direct_data->fxdata), rule_data->excase_data);
	    if (status) {
	    } else {
		rule_data->roll = 1;
		rule_data->direct_data++;
	    }
	} else {
	    int (*copy)(void *, void *, void **, const void *,
			const struct x1f4_nodelink_type *);

	    copy = lxtext_data->lxtype.copy;
	    if (copy) {
		status = copy
		    (lxtext_data->lxtype.context, NULL,
		     (void *) &direct_data->fxdata,
		     I_USER(&direct_data->fxdata), rule_data->excase_data);
		if (status) {
		} else {
		    rule_data->roll = 1;
		    rule_data->direct_data++;
		}
	    } else {
		status = 0;

		rule_data->direct_data++;
	    }
	}
    } else {
	status = 0;

	rule_data->direct_data++;
    }

    return status;
}


int
_libx1f4i0_lxdash_ever_line(struct lxrail_type *lxrail_data,
			    struct direct_type *direct_data, void *dxdash,
			    unsigned *miss)
{
    int status;
    struct rule_type rule;

    rule.direct_data = direct_data;
    rule.excase_data = lock_excase(dxdash);
    rule.lxtext_data = lxdash(lxrail_data)->link_f.text;

    rule.roll = 0;

    status = _libx1f4i0_lime_lxfset(dxdash(dxdash)->dash, &rule, rule_dash);

    if (rule.roll) {
	*miss = rule.direct_data - direct_data;
    }

    return status;
}
