/*
 * lxdata-f.d.c
 * Copyright (C) 2013-2018, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <stdint.h>

#include <e4.h>
#include <integral_q.h>
#include <lxdata-drive.h>
#include <lxdata-inter.h>
#include <lxdata-size.h>
#include <lxdata-types.h>

#if SIZEOF_UNSIGNED_LONG == 4
# define HEAD_ZERO			UINT32_C(0x80808080)
# define MISS_HOLE			UINT32_C(0x7efefeff)
# define MISS_ZERO			UINT32_C(0x01010101)
#else
# define HEAD_ZERO			UINT64_C(0x8080808080808080)
# define MISS_HOLE			UINT64_C(0x7efefefefefefeff)
# define MISS_ZERO			UINT64_C(0x0101010101010101)
#endif				/* SIZEOF_UNSIGNED_LONG == 4 */

#if WORDS_BIGENDIAN
# define LAND_MASK			(~(~0ul >> 1))
# define LEAD_SLIP(___c) \
    ((___c) >> ((sizeof(unsigned long) - 1) << 3))
# define TRAP_SLIP(___c)		(___c) <<= 8
#else
# define LAND_MASK			0x80ul
# define LEAD_SLIP(___c)		(___c)
# define TRAP_SLIP(___c)		(___c) >>= 8
#endif				/* WORDS_BIGENDIAN */

static void fail_data(unsigned char *, unsigned *, unsigned);

static void
fail_data(unsigned char *data, unsigned *fail, unsigned side)
{
    unsigned size, slip;
    unsigned char *text;

    size = *fail;

    text = data;

    slip =
	(SIZEOF_UNSIGNED_LONG
	 - ((integral_q) data & (SIZEOF_UNSIGNED_LONG - 1)))
	& (SIZEOF_UNSIGNED_LONG - 1);
    if (slip) {
	if (size < slip) {
	    slip = size;
	}

	size -= slip;

	for (; slip; slip--) {
	    int c;

	    c = *data++;
	    if (c ^ side) {
		*text++ = c;
	    }
	}
    }

    if (size) {
	slip = size / SIZEOF_UNSIGNED_LONG;
	if (slip) {
	    unsigned long *deck, hole;

            hole = side;

	    hole |= hole << 010;
	    hole |= hole << 020;
# if SIZEOF_UNSIGNED_LONG == 8
	    hole |= hole << 040;
#endif				/* SIZEOF_UNSIGNED_LONG == 8 */

	    deck = (unsigned long *) data;

	    do {
		unsigned long miss, this, trap;

		miss = *deck++;
		this = miss ^ hole;
#if 0
		trap = ((this - MISS_ZERO) & ~this) & HEAD_ZERO;
#else
		trap = ((this - ((this ^ MISS_ZERO) & MISS_ZERO)) & ~this)
		    & HEAD_ZERO;
#endif				/* 0 */
		if (trap) {
#if 0
		    unsigned i;

		    data = (unsigned char *) (deck - 1);
		    for (i = sizeof(this); i; i--) {
			int c;

			c = *data++;
			if (c ^ side) {
			    *text++ = c;
			}
		    }
#else
		    if (this) {
			trap ^= HEAD_ZERO;
			while (trap) {
			    if (trap & LAND_MASK) {
				*text++ = LEAD_SLIP(miss);
			    }
			    TRAP_SLIP(miss);
			    TRAP_SLIP(trap);
			}
		    } else {
		    }
#endif				/* 0 */
		} else {
		    unsigned long *flat;

		    flat = (unsigned long *) text;
		    *flat++ = miss;
		    text = (unsigned char *) flat;
		}

		slip--;
	    } while (slip);

	    data = (unsigned char *) deck;

	    size &= SIZEOF_UNSIGNED_LONG - 1;
	}

	for (; size; size--) {
	    int c;

	    c = *data++;
	    if (c ^ side) {
		*text++ = c;
	    }
	}
    }

    *text = 0;

    *fail -= data - text;
}


int
_libx1f4i0_lxdata_d_forward(void *context, void *output, void **input)
{
    int status;
    struct dxdata_type *dxdata_data;
    unsigned size;

    dxdata_data = I_DATA(input[0]);

    size = dxdata_data->length;
    if (size) {
	unsigned rate, side;
	void *text;

	text = dxdata_data->string;

	rate = size;

	side = I_MODE(input[1]);

	if (side < 256) {
	    fail_data(text, &rate, side);
	}

	if (rate == size) {
	    status = 0;
	} else {
	    status = _libx1f4i0_lxdata_part_data(context, dxdata_data, rate);
	}
    } else {
	status = 0;
    }

    l_DATA(dxdata_data, output);

    return status;
}
