/*
 * lxdata-p.b.c
 * Copyright (C) 2016, 2019, 2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <e4.h>
#include <lxcall.h>
#include <lxdata-drive.h>
#include <lxdata-inter.h>
#include <lxdata-types.h>

int
_libx1f4i0_lxdata_b_pending(void *context, void *output, void **input)
{
    int status;
    void *data;

    status = x1f4_line_lxdata(context, &data);
    if (status) {
    } else {
	struct lxdata_type *lxdata_data;

	lxdata_data = context;

	status = x1f4_raft_node
	    (&lxdata_data->link_v, &lxdata_data->link_e,
	     lxdata_data->link_t.datatype, data);
	if (status) {
	    _libx1f4i0_lxdata_slip_data(context, &data);
	} else {
	    const int *args;
	    struct x1f4_function_type *function_data;
	    unsigned count;

	    function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

	    args = function_data->args;

	    count = function_data->count;

	    if (count) {
		X1f4_E4_C_TEXT line;

		do {
		    char c;
		    int type;
		    void *call;

		    call = *input++;

		    type = *args++;
		    if (type == X1f4_E4_CASE) {
			struct x1f4_dxcast_type *object_data;

			object_data = I_USER(call);
			call = &object_data->data;
			type = object_data->type;
		    }

		    switch (type) {
		    case X1f4_E4_BILL:
			c = I_BILL(call);
			status = _libx1f4i0_lxdata_land_data
			    (lxdata_data, data, &c, 1);
			break;
		    case X1f4_E4_MODE:
			c = I_MODE(call);
			status = _libx1f4i0_lxdata_land_data
			    (lxdata_data, data, &c, 1);
			break;
		    case X1f4_E4_REAL:
			c = I_REAL(call);
			status = _libx1f4i0_lxdata_land_data
			    (lxdata_data, data, &c, 1);
			break;
		    case X1f4_E4_TEXT:
			line = I_TEXT(call);
			status = _libx1f4i0_lxdata_land_data
			    (lxdata_data, data, line, strlen(line));
			break;
		    default:
			if (type == lxdata_data->link_t.datatype[0].type) {
			    struct dxdata_type *dxdata_text;

			    dxdata_text = I_USER(call);
			    status = _libx1f4i0_lxdata_land_data
				(lxdata_data, data, dxdata_text->string,
				 dxdata_text->length);
			}
		    }

		    if (status) {
			break;
		    }

		    count--;
		} while (count);
	    }

	    if (status) {
	    } else {
		l_DATA(data, output);
	    }
	}
    }

    return status;
}
