/*
 * lxdata-p.x.c
 * Copyright (C) 2011-2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <it.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxdata-drive.h>
#include <lxdata-inter.h>
#include <lxdata-types.h>

extern const void *const x1f4_a1_walk_link;

static int ever_call(void **, void **, unsigned, struct dxdata_type *, int *,
		     const int *, unsigned, unsigned);

static int
ever_call(void **post_data, void **line_data, unsigned parts,
	  struct dxdata_type *dxdata_data, int *post_args,
	  const int *line_args, unsigned line, unsigned link)
{
    X1f4_E4_C_MODE *hold;
    unsigned i;
    unsigned char *p;

    hold = (void *) (post_data + line + link);

    memcpy(post_data, line_data + 2, line * sizeof(void *));

    post_data += line;

    memcpy(post_args, line_args + 2, line * sizeof(int));

    post_args += line;

    p = (void *) dxdata_data->string;

    i = dxdata_data->length;
    for (; i; i--) {
	*post_args++ = X1f4_E4_MODE;
	*post_data++ = hold;
	*hold++ = *p++;
    }

    if (parts & X1f4_E4_POST_TYPE) {
	memcpy(post_args, line_args + 4 + line, line * sizeof(int));
	post_args += line;
	if (0) {
	} else {
	    memset(post_args, 0, link * sizeof(int));
	}
    }

    return 0;
}


int
_libx1f4i0_lxdata_x_pending(void *context, void *output, void **input)
{
    int status;
    struct lxdata_type *lxdata_data;
    struct x1f4_dxcast_type *dxcast_data;
    void **track, **used;

    if (output == x1f4_a1_walk_link) {
	track = input[1];
    } else {
	track = input;
    }

    used = track[-1];

    lxdata_data = context;

    dxcast_data = I_USER(track[1]);

    if (X1f4_E4_CALL < dxcast_data->type) {
	struct x1f4_linetext_type *linetext_data;

	linetext_data = I_USER(&dxcast_data->data);
	if (linetext_data->function.function) {
	    struct dxdata_type *dxdata_data;
	    struct x1f4_function_type function;
	    const struct x1f4_function_type *function_text;
	    unsigned class, count, parts, scale = 0, trip;
	    void *beta;

	    dxdata_data = I_DATA(track[0]);

	    count = dxdata_data->length;

	    function_text = used[X1f4_E4_RENDITION];

	    class = function_text->count - 2;

	    function.count = count + class;

	    parts = function_text->flags;

	    function.flags = parts & X1f4_E4_POST_TYPE;

	    if (function.flags) {
		scale = 1;
	    }

	    trip = (sizeof(void *) + (sizeof(int) << scale)) * function.count
		+ sizeof(X1f4_E4_C_MODE) * count + sizeof(void *);

	    status = lxdata_data->link_v.link
		(lxdata_data->link_v.data, &beta, trip);
	    if (status) {
		status = _libx1f4i0_lxdata_stat_link(lxdata_data);
	    } else {
		X1f4_E4_C_MODE *line;
		void *clear[3], **frame;

		frame = beta;
		frame++;

		line = (void *) (frame + function.count);

		function.args = (void *) (line + count);

		frame[-1] = used;

		ever_call
		    (frame, track, parts, dxdata_data, (int *) function.args,
		     function_text->args, class, count);

		if (output == x1f4_a1_walk_link) {
		    clear[0] = input[0];
		    clear[1] = frame;
		    clear[2] = input[2];

		    frame = clear;
		}

		status = _libx1f4i0_lxcall_post_call
		    (linetext_data, &function, output, 0, frame,
		     X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA,
		     &lxdata_data->link_v, &lxdata_data->link_p,
		     &lxdata_data->link_q, &lxdata_data->link_e,
		     lxdata_data->trap_e.eelookup_data);
		if (status) {
		} else {
		    if (output == x1f4_a1_walk_link) {
			input[2] = clear[2];
		    }
		}
	    }
	} else {
	    status = _libx1f4i0_lxdata_stat_zero(lxdata_data);
	}
    } else {
	status = _libx1f4i0_lxdata_stat_roll(lxdata_data, "b_xcall", 7);
    }

    return status;
}
