/*
 * lxdecq-a.j.c
 * Copyright (C) 2017-2019, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <e4.h>
#include <lxdecq-types.h>
#include <progress.h>

#define lxdecq(lxdecq) \
    ((struct lxdecq_type *) (lxdecq))

static const struct x1f4_progress_type progress_1b = {
/* *INDENT-OFF* */
    {	"",				1,	1,	1,	19	},
    {	"",				1,	1,	2,	29	},
	X1f4_E4_MODE,	X1f4_E4_SLIP,	0,
	X1f4_PROGRESS_BACKWARD,		2,
	NULL
/* *INDENT-ON* */
}, progress_1f = {
/* *INDENT-OFF* */
    {	"",				1,	1,	1,	19	},
    {	"",				1,	1,	2,	29	},
	X1f4_E4_MODE,	X1f4_E4_SLIP,	0,
	0,				2,
	NULL
/* *INDENT-ON* */
};

static int
pass_type(struct x1f4_progress_type *progress_data, char **pass,
	  struct lxtext_type *lxtext_data, int d)
{
    char *text;
    unsigned size;

    size = lxtext_data->lxtype.size;

    progress_data->arg2 = lxtext_data->lxtype.code;

    text = *pass;

    progress_data->do_clause.size = size + 19 + 8;
    progress_data->do_clause.text = text;

    memcpy(text, "(;0 -= 1) && q._js_", 19);
    text[14] = d;
    text += 19;
    memcpy(text, lxtext_data->lxtype.name, size);
    text += size;
    memcpy(text, "(;1, ;*)", 8);
    text += 8;

    progress_data->if_clause.size = size + 29 + 8;
    progress_data->if_clause.text = text;

    memcpy(text, "(;0 = q_length(;*)) && q._js_", 29);
    text[24] = d;
    text += 29;
    memcpy(text, lxtext_data->lxtype.name, size);
    text += size;
    memcpy(text, "(;1, ;*)", 8);
    text += 8;

    *pass = text;

    return 0;
}


int
x1f4_pass_lxdecq(void *lxdecq, struct x1f4_progress_type **progress,
		 char **copy)
{
    char *text;
    int type;
    struct lxtext_type *lxtext_data;
    struct x1f4_progress_type *progress_data;
    unsigned miss;

    type = lxdecq(lxdecq)->link_t.datatype[0].type;

    progress_data = *progress;

    text = *copy;

    lxtext_data = lxdecq(lxdecq)->link_f.text;

    miss = ((struct lxmiss_type *) lxtext_data - 1)->data.miss;
    for (; miss; miss--) {
	lxtext_data++;

	if (lxtext_data->lxtype.flags & X1f4_LX_TYPE_EXCEPT) {
	} else {
	    *progress_data = progress_1f;
	    progress_data->type = type;
	    pass_type(progress_data, &text, lxtext_data, 'f');

	    progress_data++;

	    *progress_data = progress_1b;
	    progress_data->type = type;
	    pass_type(progress_data, &text, lxtext_data, 'b');

	    progress_data++;
	}
    }

    *copy = text;

    *progress = progress_data;

    return 0;
}
