/*
 * lxfile-p.r.c
 * Copyright (C) 2008-2010, 2013, 2014, 2016, 2018, 2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <c1.h>
#include <e4.h>
#include <file.h>
#include <lxdata.h>
#include <lxfile-defs.h>
#include <lxfile-drive.h>
#include <lxfile-inter.h>
#include <lxfile-types.h>
#include <trans.h>

int
_libx1f4i0_lxfile_r_pending(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_MODE mode;
	struct dxfile_type *dxfile_data;
	struct lxfile_type *lxfile_data;
	struct trans_type trans;
	struct x1f4_dxdata_type *dxdata_data;
	unsigned size;
	void *data, *file;

	dxfile_data = I_FILE(input[0]);

	lxfile_data = context;

	file = dxfile_data->file;

	if (dxfile_data->mode != READ_CLASS) {
	    status = _libx1f4i0_lxfile_stat_turn(lxfile_data, dxfile_data);
	    if (1) {
		break;
	    }
	}

	mode = I_MODE(input[2]);
	if (mode < 0) {
	    status = _libx1f4i0_lxfile_stat_rate(lxfile_data, mode);
	    if (1) {
		break;
	    }
	}

	x1f4_hold_lxdata(lxfile_data->link_l.data, (void *) &trans);

	dxdata_data = I_DATA(input[1]);

	if (mode) {
	    if (dxdata_data->length != mode) {
		if (dxdata_data->string != x1f4_c1_empty_string) {
		    status = trans.free(trans.data, dxdata_data->string);
		    if (status) {
			status = _libx1f4i0_lxfile_stat_free(lxfile_data, 0);
			if (1) {
			    break;
			}
		    }
		}

		status = trans.link(trans.data, &data, mode + 1);
		if (status) {
		    status = _libx1f4i0_lxfile_stat_link(lxfile_data);

		    dxdata_data->length = 0;
		    dxdata_data->string = (char *) x1f4_c1_empty_string;

		    if (1) {
			break;
		    }
		}

		dxdata_data->length = mode;
		dxdata_data->string = data;
	    } else {
		data = dxdata_data->string;
	    }
	} else {
	    status = 0;

	    l_MODE(0, output);

	    if (dxdata_data->string != x1f4_c1_empty_string) {
		status = trans.free(trans.data, dxdata_data->string);
		if (status) {
		    status = _libx1f4i0_lxfile_stat_free(lxfile_data, 0);
		} else {
		    dxdata_data->length = 0;
		    dxdata_data->string = (char *) x1f4_c1_empty_string;
		}
	    }

	    if (1) {
		break;
	    }
	}

	status = _libx1f4i0_read_ifile
	    (file, &data, &size, mode, 0, &trans);
	if (status) {
	    if (status == FILE_NONE_CLASS) {
		status = 0;
		size = 0;

		l_MODE(0, output);
	    } else {
		if (status == FILE_OVER_CLASS) {
		    status = 0;
		    size = 0;

		    l_MODE(-1, output);
		} else {
		    status = _libx1f4i0_lxfile_stat_file(lxfile_data, status);
		    if (1) {
			break;
		    }
		}
	    }
	} else {
	    l_MODE(size, output);

	    dxdata_data->length = size;
	    dxdata_data->string = data;
	}

	x1f4_kate_lxdata(lxfile_data->link_l.data, dxdata_data, 0, size);
    } while (0);

    return status;
}
