/*
 * lxlist-d.y.c
 * Copyright (C) 2008-2011, 2013-2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <copy.h>
#include <e4.h>
#include <fd.h>
#include <l_list.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#if __HAVE_PULL_L4LIST__
# define clip(none) \
    ((struct clip_type *) (none))
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
typedef struct clip_type {
    X1f4_E4_C_MODE mode;
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    unsigned call, type;
    void *effect;
} clip_type;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int clip_miss(void *, void *);
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int
clip_miss(void *clip, void *data)
{
    int status;
    struct lxlist_type *lxlist_data;
    unsigned link, type;

    lxlist_data = clip(clip)->lxlist_data;

    type = effect_type(data);
    if (type != clip(clip)->call) {
	clip(clip)->status = _libx1f4i0_lxlist_stat_type
	    (lxlist_data, clip(clip)->mode, type, clip(clip)->call);

	status = 1;
    } else {
	link = effect_link(data);

	do {
	    if (~link) {
		struct lxtext_type *lxtext_data;

		lxtext_data = lxlist_data->link_f.text;
		lxtext_data += link;
		if (lxtext_data->lxtype.push) {
		    status = lxtext_data->lxtype.push
			(lxtext_data->lxtype.context, clip(clip)->effect,
			 E_USER(data),
			 lock_excase(clip(clip)->dxlist_data));
		    if (status) {
			clip(clip)->status = status;
			if (1) {
			    break;
			}
		    }
		} else {
		    copy_miss(clip(clip)->effect, type, effect_data(data));
		}

		link_effect(data, ~0);
	    } else {
		copy_miss(clip(clip)->effect, type, effect_data(data));
	    }

	    clip(clip)->type = type;

	    status = 0;
	} while (0);
    }

    return status;
}
#endif				/* __HAVE_PULL_L4LIST__ */


int
_libx1f4i0_lxlist_y_derived(void *context, void *output, void **input)
{
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;

    lxlist_data = context;

    dxlist_data = I_LIST(input[1]);
    if (0) {
    } else {
	X1f4_E4_C_MODE mode;
	unsigned size;

	mode = I_MODE(input[2]);

	size = dxlist_data->size;
	if (mode < -(X1f4_E4_C_MODE) (int) size
	    || !(mode < (X1f4_E4_C_MODE) (int) size)) {
	    status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);
	} else {
#if __HAVE_PULL_L4LIST__
	    struct clip_type clip;
#else
	    void *data;
#endif				/* __HAVE_PULL_L4LIST__ */

	    if (mode < 0) {
		mode += size;
	    }

#if __HAVE_PULL_L4LIST__
	    clip.call = *((struct x1f4_function_type *)
			  ((void **) input[-1])[X1f4_E4_RENDITION])->args;
	    clip.dxlist_data = dxlist_data;
	    clip.lxlist_data = context;
	    clip.effect = *input;
	    clip.mode = mode;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
	    status = _libx1f4i0_pull_l4list
		(dxlist_data->list, mode, &clip, clip_miss);
	    if (status) {
		if (status == X1f4_L4LIST_CALL_ERROR) {
		    status = clip.status;
		} else {
		    status = _libx1f4i0_lxlist_stat_list
			(lxlist_data, 0, status);
		}
	    } else {
		dxlist_data->size = size - 1;

		status = _libx1f4i0_lxrail_cast_data
		    (lxlist_data, output, clip.type, *input);
	    }
#else
	    status = _libx1f4i0_peek_l4list(dxlist_data->list, mode, &data);
	    if (status) {
		status = _libx1f4i0_lxlist_stat_list(lxlist_data, 0, status);
	    } else {
		unsigned call, type;

		type = effect_type(data);
		call = *((struct x1f4_function_type *)
			 ((void **) input[-1])[X1f4_E4_RENDITION])->args;
		if (type != call) {
		    status = _libx1f4i0_lxlist_stat_type
			(lxlist_data, mode, type, call);
		} else {
		    unsigned link;

		    link = effect_link(data);

		    do {
			if (~link) {
			    struct lxtext_type *lxtext_data;

			    lxtext_data = lxlist_data->link_f.text;
			    lxtext_data += link;
			    if (lxtext_data->lxtype.push) {
				status = lxtext_data->lxtype.push
				    (lxtext_data->lxtype.context,
				     *input, E_USER(data),
				     lock_excase(dxlist_data));
				if (status) {
				    break;
				}
			    } else {
				copy_miss(*input, type, effect_data(data));
			    }

			    link_effect(data, ~0);
			} else {
			    copy_miss(*input, type, effect_data(data));
			}

			status = _libx1f4i0_miss_l4list
			    (dxlist_data->list, mode);
			if (status) {
			    status = _libx1f4i0_lxlist_stat_list
				(lxlist_data, 0, status);
			} else {
			    dxlist_data->size = size - 1;

			    status = _libx1f4i0_lxrail_cast_data
				(lxlist_data, output, type, *input);
			}
		    } while (0);
		}
	    }
#endif				/* __HAVE_PULL_L4LIST__ */
	}
    }

    return status;
}
