/*
 * lxlist-i.f.c
 * Copyright (C) 2011-2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e4.h>
#include <fd.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#define still				args

#define POST_BITS \
    X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA | X1f4_LXCALL_POST_LINE

#define lxlist(list) \
    ((struct lxlist_type *) (list))

typedef struct pick_type {
    struct fxdata_type finite;
    struct x1f4_dxcast_type dxcast, serial;
    struct x1f4_function_type function;
    struct x1f4_linetext_type linetext;
} pick_type;

/*
 * function does setup for an unused output
 */
int
_libx1f4i0_lxlist_fold_call(void *lxlist, void *output, void **input,
			    struct x1f4_linetext_type *linetext_data,
			    unsigned count,
			    const struct x1f4_function_type *function_text,
			    int lead, int lock, int *press,
			    struct fxdata_type *fxdata_data)
{
    int linked, status;
    unsigned class, grade, parts, scale = 0;
    void *beta;

    class = function_text->count - lead;

    grade = class + 1;

    parts = function_text->flags;

    if (parts & X1f4_E4_POST_TYPE) {
	if (linetext_data->function.flags
	    & (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
	    scale = 1;
	} else {
	    parts &= ~X1f4_E4_POST_TYPE;
	}
    }

    linked = sizeof(struct pick_type);

    status = lxlist(lxlist)->link_v.link
	(lxlist(lxlist)->link_v.data, &beta,
	 linked + (sizeof(void *) + (sizeof(int) << scale)) * grade
	 + sizeof(void *));
    if (status) {
	status = _libx1f4i0_lxlist_stat_link(lxlist);
    } else {
	int *args, type;
	struct pick_type *call_data;
	struct x1f4_function_type *function_call;
	unsigned bits;
	void **frame, **plane;

	call_data = beta;

	function_call = &call_data->function;

	function_call->count = grade;

	function_call->flags = parts & X1f4_E4_POST_TYPE;

	frame = (void *) (call_data + 1);
	frame++;

	function_call->args = args = (void *) (frame + grade);

	frame[-1] = input[-1];

	type = linetext_data->function.type;

	if (type != X1f4_E4_CASE) {
	    l_USER(&call_data->dxcast, output);

	    call_data->dxcast.call = 0;
	    call_data->dxcast.type = type;

	    output = &call_data->dxcast.data;
	} else {
	}

	bits = POST_BITS;

	_libx1f4i0_lxcall_hold_call
	    (frame, input, parts, args, function_text->args, class, lead,
	     lock);

	args += lock;
	plane = frame + lock;

	*still = *press;
	*plane = fxdata_data;

	{
	    /* the args, count, flags set should do */
	    call_data->linetext = *linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    status = _libx1f4i0_lxcall_post_call
		(&call_data->linetext, function_call, output, 0, frame, bits,
		 &lxlist(lxlist)->link_v, &lxlist(lxlist)->link_p,
		 &lxlist(lxlist)->link_q, &lxlist(lxlist)->link_e,
		 lxlist(lxlist)->trap_e.eelookup_data);
	    if (status) {
		count = 0;
	    } else {
		count--;
	    }
	}
	for (; count; count--) {
	    fxdata_data++;

	    press += 2;

	    *still = *press;
	    *plane = fxdata_data;

	    /* the args, count, flags set should do */
	    call_data->linetext = *linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    status = _libx1f4i0_lxcall_rail_call
		(&call_data->linetext, function_call, output, 0, frame, bits,
		 &lxlist(lxlist)->link_p, &lxlist(lxlist)->link_q,
		 &lxlist(lxlist)->link_e, lxlist(lxlist)->trap_e.eelookup_data,
		 1, &lock, &call_data->finite, &call_data->serial);
	    if (status) {
		break;
	    }
	}
    }

    return status;
}
