/*
 * lxlist-k.n.c
 * Copyright (C) 2008-2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <copy.h>
#include <e4.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

int
_libx1f4i0_lxlist_n_keeping(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_MODE mode, take;
	const int *args;
	int sign;
	struct cxbend_type *cxbend_data;
	struct dxlist_type *dxlist_data;
	struct lxlist_type *lxlist_data;
	struct x1f4_function_type *function_data;
	unsigned count, i, size;
	void *data, **track;

	dxlist_data = I_LIST(input[0]);

	cxbend_data = context;

	function_data = (void *) ((void **) input[-1])[X1f4_E4_RENDITION];

	mode = I_MODE(input[1]);

	count = function_data->count;

	take = count - 2;

	sign = cxbend_data->sign;

	size = dxlist_data->size;
	if (mode + take * sign < -(X1f4_E4_C_MODE) (int) size
	    || (X1f4_E4_C_MODE) (int) size < mode + take * (1 + sign)) {
	    l_MODE(mode + take * sign, input[1]);
	    l_MODE(0, output);

	    status = 0;

	    break;
	}

	l_MODE(1, output);

	lxlist_data = cxbend_data->list;

	args = function_data->args + 2;

	track = input + 2;

	mode += take * sign;

	i = take;
	for (; i; i--, mode++, args++, track++) {
	    int pick;
	    unsigned call, type;

	    pick = mode;
	    if (pick < 0) {
		pick += size;
	    }

	    status = _libx1f4i0_peek_l4list(dxlist_data->list, pick, &data);
	    if (status) {
		status = _libx1f4i0_lxlist_stat_list(lxlist_data, 0, status);
		if (1) {
		    break;
		}
	    }

	    type = effect_type(data);
	    call = *args;
	    if (type != call) {
		status = _libx1f4i0_lxlist_stat_type
		    (lxlist_data, mode, type, call);
		if (1) {
		    break;
		}
	    }

	    if (args[count] & X1f4_E4_POST_XSET) {
		unsigned link;

		link = effect_link(data);

		do {
		    if (~link) {
			struct lxtext_type *lxtext_data;

			lxtext_data = lxlist_data->link_f.text;
			lxtext_data += link;
			if (lxtext_data->lxtype.lead
			    && lxtext_data->lxtype.slip) {
			    status = _libx1f4i0_lxcall_land_node
				(&lxlist_data->link_v, &lxlist_data->link_e,
				 &lxtext_data->lxtype, E_USER(data), *track);

			    break;
			}

			if (lxtext_data->lxtype.copy
			    && lxtext_data->lxtype.push) {
			    status = _libx1f4i0_lxcall_side_node
				(&lxtext_data->lxtype, E_USER(data), *track);

			    break;
			}
		    }

		    copy_miss(*track, type, effect_data(data));
		} while (0);
		if (status) {
		    break;
		}
	    }
	}

	if (status) {
	} else {
	    l_MODE(mode - take, input[1]);
	}
    } while (0);

    return status;
}
