/*
 * lxlist-l.r.c
 * Copyright (C) 2008-2011, 2013, 2014, 2017, 2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxlist-config.h>

#include <stddef.h>

#include <copy.h>
#include <e4.h>
#include <l_list.h>
#include <lxcast.h>
#include <lxlist-close.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>
#include <nt.h>

int
_libx1f4i0_lxlist_r_logique(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    unsigned size;

    dxlist_data = I_LIST(input[0]);

    lxlist_data = context;

    mode = I_MODE(input[1]);

    size = dxlist_data->size;
    if (mode < -(X1f4_E4_C_MODE) (int) size
	|| !(mode < (X1f4_E4_C_MODE) (int) size)) {
	status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);
    } else {
	int type;
	struct lxtext_type *lxtext_data, *lxtext_link;
	unsigned miss;
	void *trap;

	trap = input[2];

	type = ((struct x1f4_function_type *)
		((void **) input[-1])[X1f4_E4_RENDITION])->args[2];
	if (type == X1f4_E4_CASE) {
	    struct x1f4_dxcast_type *dxcast_data;

	    dxcast_data = I_USER(trap);
	    trap = &dxcast_data->data;
	    type = dxcast_data->type;
	}

	lxtext_link = lxlist_data->link_f.text;

	lxtext_data = lxtext_link;

	close_frame(miss, lxtext_data);

	do {
	    int (*copy) (void *, void *, void **, const void *,
			 const struct x1f4_nodelink_type *);
	    void *data, *side;

	    if (miss) {
		copy = lxtext_data->lxtype.copy;
		if (copy) {
		    status = copy
			(lxtext_data->lxtype.context, NULL,
			 &side, I_USER(trap), lock_excase(dxlist_data));
		    if (status) {
			break;
		    }
		}
	    } else {
		copy = NULL;
	    }

	    if (mode < 0) {
		mode += size;
	    }

	    status = _libx1f4i0_peek_l4list(dxlist_data->list, mode, &data);
	    if (status) {
		status = _libx1f4i0_lxlist_stat_list(lxlist_data, 0, status);

		if (copy) {
		    lxtext_data->lxtype.free
			(lxtext_data->lxtype.context, side,
			 NULL, lock_excase(dxlist_data));
		}
	    } else {
		if (1) {
		    unsigned link;

		    link = effect_link(data);
		    if (~link) {
			int (*free)(void *, void *, struct excase_type **,
				    const struct excase_type *);
			struct lxtext_type *lxtext_slip;

			lxtext_slip = lxtext_link + link;

			free = lxtext_slip->lxtype.free;
			if (free) {
			    status = free
				(lxtext_slip->lxtype.context, E_USER(data),
				 NULL, lock_excase(dxlist_data));
			}
		    }
		}
		{
		    unsigned link;

		    type_effect(data, type);

		    link = miss ? lxtext_data - lxtext_link : ~0;

		    link_effect(data, link);

		    if (copy) {
			l_USER(side, effect_data(data));
		    } else {
			copy_miss(effect_data(data), type, trap);
		    }
		}
	    }
	} while (0);
    }

    return status;
}
