/*
 * lxlist-t.p.f.c
 * Copyright (C) 2008-2011, 2013, 2014, 2017-2020, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <e4.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

int
_libx1f4i0_lxlist_pfclass_t(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_TEXT text;
	struct dxlist_type *dxlist_data;
	struct lxlist_type *lxlist_data;
	struct lxtext_type *lxtext_data;
	void *slip;

	lxtext_data = context;

	lxlist_data = lxtext_data->text;

	dxlist_data = I_LIST(input[0]);

	text = I_TEXT(input[1]);

	status = lxtext_data->lxtype.copy
	    (lxtext_data->lxtype.context, NULL, &slip, text,
	     lock_excase(dxlist_data));
	if (status) {
	    break;
	} else {
	    void *data;

#if defined __MISSING_HEAD_L_LIST__
	    status = _libx1f4i0_push_l4list
		(dxlist_data->list, 0, &data);
#else
	    status = _libx1f4i0_head_l4list(dxlist_data->list, &data);
#endif				/* __MISSING_HEAD_L_LIST__ */
	    if (status) {
		status = _libx1f4i0_lxlist_stat_list
		    (lxtext_data->text, 0, status);

		lxtext_data->lxtype.free
		    (lxtext_data->lxtype.context,
		     slip, NULL, lock_excase(dxlist_data));
	    } else {
#define type				X1f4_E4_TEXT

		unsigned link;

		type_effect(data, type);

		link = lxtext_data
		    - ((struct lxtext_type *) lxlist_data->link_f.text);

		link_effect(data, link);

		dxlist_data->size++;

		l_TEXT(slip, effect_data(data));

#undef type

#if 1
		l_TEXT(text, output);
#else
		status = x1f4_auto_text
		    (&lxlist_data->link_v, &lxlist_data->link_e, slip,
		     strlen(slip), output);
#endif				/* 1 */
	    }
	}
    } while (0);

    return status;
}

