/*
 * lxmile-e.c.c
 * Copyright (C) 2008-2015, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <exerrors.h>
#include <lxlead.h>
#include <lxmile-inter.h>
#include <lxmile-types.h>
#include <nl.h>
#include <tcvset.h>

#define tccase(vset) \
    ((struct x1f4_tccase_type *) (vset))

int
x1f4_free_lxmile(void *subtext, void *dxpart,
		 struct x1f4_nodelink_type **nodelink,
		 const struct x1f4_nodelink_type *nodelink_data)
{
    int status;
    unsigned call;

    call = lock_expart(dxpart)->call;

    /*
     * FEATURES
     *
     * some code will work even without this test, but some will not.  among
     * the one that will not is _long_mile/_near_part (some reasons recorded
     * there).
     *
     * (old note by now, might be meaningless)
     */
    if (call) {
	call--;
	if (call) {
	    struct lxmile_type *lxmile_data;

	    lxmile_data = ((struct lxtext_type *) subtext)->part;

	    status = tccase(lock_expart(dxpart)->text)->miss
		(&lock_expart(dxpart)->pset, &lock_expart(dxpart)->text,
		 nodelink_data, (void *) &lxmile_data->link_m);
	    if (status) {
		status = X1f4_EX_CRITICAL;
	    } else {
		int (*miss)(void *, struct x1f4_nodelink_type **,
			    struct x1f4_nodelink_type *);

		lock_expart(dxpart)->call = call;

		miss = lxmile_data->link_r.miss;
		if (miss) {
		    status = miss
			(lxmile_data->link_r.text, nodelink,
			 lock_excase(dxpart));
		} else {
		    status = x1f4_mind_lxlead(nodelink, dxpart);
		}
	    }
	} else {
	    status = _libx1f4i0_lxmile_heap_mile(subtext, nodelink, dxpart);
	}
    } else {
	status = 0;
    }

    return status;
}
