/*
 * lxmode-s.h.c
 * Copyright (C) 2011-2017, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <fd.h>
#include <it.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxmode-inter.h>
#include <lxmode-types.h>

#define POST_BITS \
    X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA | X1f4_LXCALL_POST_LINE

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define lxmode(dash) \
    ((struct lxmode_type *) (dash))

typedef struct pick_type {
    struct fxdata_type finite[1];
    struct x1f4_dxcast_type dxcast, serial[1];
    struct x1f4_function_type function;
    struct x1f4_linetext_type linetext;
} pick_type;

/*
 * function does setup for an unused output
 */
int
_libx1f4i0_lxmode_jack_call(void *lxmode, void *output, void **input,
			    struct x1f4_linetext_type *linetext_data,
			    unsigned count,
			    const struct x1f4_function_type *function_text,
			    int lead, int jack, int over, void *fydata)
{
    int linked, status;
    unsigned class, grade, parts, scale = 0;
    void *beta;

    class = function_text->count - lead;

    grade = class + 1;

    parts = function_text->flags;

    if (parts & X1f4_E4_POST_TYPE) {
	if (linetext_data->function.flags
	    & (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
	    scale = 1;
	} else {
	    parts &= ~X1f4_E4_POST_TYPE;
	}
    }

    linked = sizeof(struct pick_type);

    status = lxmode(lxmode)->link_v.link
	(lxmode(lxmode)->link_v.data, &beta,
	 linked + (sizeof(void *) + (sizeof(int) << scale)) * grade
	 + sizeof(void *));
    if (status) {
	status = _libx1f4i0_lxmode_stat_link(lxmode);
    } else {
	int *args, *reach, type;
	struct fydata_type *fydata_data;
	struct pick_type *call_data;
	struct x1f4_composer_type nc;
	struct x1f4_function_type *function_call;
	struct x1f4_iturn_type ni;
	unsigned bits;
	void **frame, **order;

	call_data = beta;

	function_call = &call_data->function;

	function_call->count = grade;

	function_call->flags = parts & X1f4_E4_POST_TYPE;

	frame = (void *) (call_data + 1);
	frame++;

	function_call->args = args = (void *) (frame + grade);

	frame[-1] = input[-1];

	type = linetext_data->function.type;

	if (type != X1f4_E4_CASE) {
	    l_USER(&call_data->dxcast, output);

	    call_data->dxcast.call = 0;
	    call_data->dxcast.type = type;

	    output = &call_data->dxcast.data;
	} else {
	}

	bits = POST_BITS;

	_libx1f4i0_lxcall_hold_call
	    (frame, input, parts, args, function_text->args, class, lead,
	     jack);

	args[jack] = over;

	reach = args + jack;

	order = frame + jack;

	fydata_data = fydata;

	*order = fydata_data;

	nc.link = NULL;
	ni.link = NULL;

	{
	    /* the args, count, flags set should do */
	    call_data->linetext = *linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    status = _libx1f4i0_lxcall_post_call
		(&call_data->linetext, function_call, output, 0, frame, bits,
		 &lxmode(lxmode)->link_v, &ni, &nc, &lxmode(lxmode)->link_e,
		 lxmode(lxmode)->trap_e.eelookup_data);
	    if (status) {
		count = 0;
	    } else {
		count--;
	    }
	}
	for (; count; count--) {
	    fydata_data++;

	    *order = fydata_data;
	    *reach = over;

	    /* the args, count, flags set should do */
	    call_data->linetext = *linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    status = _libx1f4i0_lxcall_rail_call
		(&call_data->linetext, function_call, output, 0, frame, bits,
		 &ni, &nc, &lxmode(lxmode)->link_e,
		 lxmode(lxmode)->trap_e.eelookup_data, 1, &jack,
		 call_data->finite, call_data->serial);
	    if (status) {
		break;
	    }
	}
    }

    return status;
}
