/*
 * lxnear-a.e.c
 * Copyright (C) 2013-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <cardinal.v-d.h>
#include <e4.h>
#include <er.h>
#include <exerrors.h>
#include <lxnear-types.h>

#define lxnear(lxnear) \
    ((struct lxnear_type *) (lxnear))

int
_libx1f4i0_lxnear_stat_bend(void *lxnear, unsigned lift)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("negative option in format specification (", 41);
    PUSH_BILL(lift);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_dash(void *lxnear, unsigned lift)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("non integral convertible option in format specification (", 57);
    PUSH_BILL(lift);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_flat(void *lxnear, unsigned lock, int news)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("bad type for argument (", 23);
    PUSH_BILL(lock);
    PUSH_DATA(") corresponding `%", 18);
    PUSH_BYTE(news);
    PUSH_DATA("' directive (`cardinal', `integer' or `real' expected)", 54);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_lead(void *lxnear, int lead)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("negative digits indication (", 28);
    PUSH_MODE(lead);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_lift(void *lxnear, unsigned lift, unsigned class)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("option index in format specification (", 38);
    PUSH_BILL(lift);
    PUSH_DATA(") outside range ( .. ", 21);
    PUSH_BILL(class - 1);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_lock(void *lxnear, unsigned lock, unsigned class)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("item index in format specification (", 36);
    PUSH_BILL(lock);
    PUSH_DATA(") outside range ( .. ", 21);
    PUSH_BILL(class - 1);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_news(void *lxnear, int news)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("unrecognized formatting directive `", 35);
    PUSH_BYTE(news);
    PUSH_DATA("'", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_norm(void *lxnear, long mode)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("bad size (", 10);
    PUSH_LONG(mode);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_post(void *lxnear)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("unmatched leading `/'", 21);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_text(void *lxnear, unsigned lock)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("bad type for argument (", 23);
    PUSH_BILL(lock);
    PUSH_DATA(") corresponding `%s' directive (`text' expected)", 48);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxnear_stat_time(void *lxnear, unsigned size, long mode)
{
    LINE_EEER(lxnear(lxnear)->link_e);

    PUSH_DATA("out of valid range (-", 21);
    PUSH_BILL(size);
    PUSH_DATA(" .. ", 4);
    PUSH_BILL(size - 1);
    PUSH_DATA(") position (", 12);
    PUSH_LONG(mode);
    PUSH_DATA(")", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}
