/*
 * lxpoll-f.p.c
 * Copyright (C) 2014, 2015, 2017, 2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <sys/time.h>

#include <d_e101.h>
#include <e4.h>
#include <lxcast.h>
#include <lxdate.h>
#include <lxpoll-defs.h>
#include <lxpoll-drive.h>
#include <lxpoll-inter.h>
#include <lxpoll-types.h>

int
_libx1f4i0_lxpoll_p_forward(void *context, void *output, void **input)
{
    int status;
    struct lxpoll_type *lxpoll_data;
    struct x1f4_dxcast_type *dxcast_data;

    lxpoll_data = context;

    dxcast_data = I_CAST(input[2]);

    if (X1f4_E4_CALL < dxcast_data->type) {
	struct dxpoll_type *dxpoll_data;
	struct x1f4_dxdate_type *dxdate_data;
	void *class[3], **pick;

	dxpoll_data = I_POLL(input[0]);

	dxdate_data = I_DATE(input[1]);

	status = _libx1f4i0_lxpoll_node_date
	    (lxpoll_data, class, dxdate_data, dxpoll_data, &pick);
	if (status) {
	} else {
	    struct x1f4_linetext_type *linetext_text;

	    linetext_text = I_USER(&dxcast_data->data);

	    status = _libx1f4i0_lxpoll_fast_line
		(linetext_text, ((void **) input[-1])[X1f4_E4_RENDITION], 3,
		 &input, dxpoll_data, lxpoll_data);
	    if (status) {
		_libx1f4i0_lxpoll_free_line
		    (lxpoll_data, NULL, class, dxpoll_data);
	    } else {
		struct Mylene101_agenda_type agenda;
		struct timeval effect;

		effect.tv_sec = dxdate_data->time;
		effect.tv_usec = dxdate_data->second_f;

		agenda.fini = _libx1f4i0_lxpoll_flat_line;
		agenda.fare = _libx1f4i0_lxpoll_fare_trap;
		agenda.text = input;

		input[NODE_LEAD + 0] = class[0];
		input[NODE_LEAD + 1] = class[1];
		input[NODE_LEAD + 2] = class[2];

		/*
		 * will not record agenda, as by the time this is examined it
		 * may have been destroyed.
		 */

		*pick = input;

		status = Mylene101_push_agenda
		    (dxpoll_data->device, &effect, &agenda, input + PICK_LEAD);
		if (status) {
		    status = _libx1f4i0_lxpoll_stat_poll
			(lxpoll_data, 0, dxpoll_data, status);

		    _libx1f4i0_lxpoll_flat_line(NULL, input);
		} else {
		    unsigned bits;

		    bits = (integral_q) input[BITS_LEAD];

		    input[BITS_LEAD] =
			(void *) (integral_q) (bits | HALF_LINE);
		}
	    }
	}
    } else {
	status = _libx1f4i0_lxpoll_stat_call(lxpoll_data);
    }

    return status;
}
