/*
 * lxport-i.u.c
 * Copyright (C) 2006-2014, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <signal.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <lxport-inter.h>
#include <lxport-types.h>

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#define SECOND_a(v)			((v).tv_sec)
#define SECOND_f(v)			((v).tv_usec)

#if defined HAVE_PSELECT
# define Select_f(v)			((v) * 1000)
#else
# define Select_f(v)			(v)
#endif				/* HAVE_PSELECT */

#if defined HAVE_PSELECT
# define Second_F(v)			((v)->tv_nsec)
#else
# define Second_F(v)			((v)->tv_usec)
#endif				/* HAVE_PSELECT */

#if defined HAVE_PSELECT
# define Second_f(v)			((v).tv_nsec)
#else
# define Second_f(v)			((v).tv_usec)
#endif				/* HAVE_PSELECT */

#define WAIT_PID() \
    {									      \
	int effect;							      \
									      \
	wait = waitpid(pend, &effect, WNOHANG);				      \
	if (wait == -1) {						      \
	    status = _libx1f4i0_lxport_stat_call(lxport, "waitpid", 7);	      \
	    break;							      \
	} else {							      \
	    if (wait) {							      \
		status = _libx1f4i0_lxport_lose_port			      \
		    (lxport, waitlink_data, &effect);			      \
		if (1) {						      \
		    break;						      \
		}							      \
	    }								      \
	}								      \
    }

int
_libx1f4i0_lxport_rend_port(void *lxport, struct waitlink_type *waitlink_data,
			    unsigned window_a, unsigned window_f)
{
    int status;
#if defined HAVE_PSELECT
    sigset_t setc, setv;
#endif				/* HAVE_PSELECT */
    struct timeval now;
    unsigned second_a, second_f;

    gettimeofday(&now, NULL);

    second_a = SECOND_a(now) + window_a;
    second_f = SECOND_f(now) + window_f;
    if (second_f < 1000000) {
    } else {
	second_a++;
	second_f -= 1000000;
    }

#if defined HAVE_PSELECT
    sigemptyset(&setc);
    sigaddset(&setc, SIGCHLD);
#endif				/* HAVE_PSELECT */

#if defined HAVE_PSELECT
    if (sigprocmask(SIG_BLOCK, &setc, &setv)) {
	status = _libx1f4i0_lxport_stat_call(lxport, "sigprocmask", 11);
    } else {
	int delete;

	delete = sigismember(&setv, SIGCHLD);
	if (delete) {
	    sigdelset(&setv, SIGCHLD);
	}
#endif				/* HAVE_PSELECT */

	status = 0;

	do {
	    pid_t pend, wait;

	    pend = waitlink_data->port;

	    WAIT_PID();

	    while (1) {
#if defined HAVE_PSELECT
		struct timespec *selector, timeout;
#else
		struct timeval *selector, timeout;
#endif				/* HAVE_PSELECT */

		gettimeofday(&now, NULL);
		if (second_a < SECOND_a(now)) {
		    break;
		} else {
		    SECOND_a(timeout) = 0;
		    if (second_a ^ SECOND_a(now)) {
			Second_f(timeout) = second_f;
			if (second_f < SECOND_f(now)) {
			    Second_f(timeout) += 1000000;
			} else {
			}

			Second_f(timeout) -= SECOND_f(now);
			Second_f(timeout) = Select_f(Second_f(timeout));
		    } else {
			if (SECOND_f(now) < second_f) {
			    Second_f(timeout) =
				Select_f(second_f - SECOND_f(now));
			} else {
			    break;
			}
		    }
		}

		/*
		 * one race condition here: the CHLD signal may arrive before
		 * the select gets a chance to observe it.  Will not let select
		 * to wait indefinitely (more than 1ms seems excessive).
		 * I expect a small likelyhood that the race will turn
		 * unfavorable and that the extra sleep will not be taken
		 * often.
		 * 
		 * The precaution should not be required for pselect(2).  But
		 * with many pselect(3)s implemented by C libraries over
		 * select(2), one never knows.
		 */
		selector = &timeout;

		if (Select_f(1000) < Second_F(selector)) {
		    Second_F(selector) = Select_f(1000);
		}

#if defined HAVE_PSELECT
		status = pselect(0, NULL, NULL, NULL, selector, &setv);
#else
		status = select(0, NULL, NULL, NULL, selector);
#endif				/* HAVE_PSELECT */
		if (status == -1) {
		    if (errno ^ EINTR) {
			status =
			    _libx1f4i0_lxport_stat_call(lxport, "select", 6);
			break;
		    } else {
			status = 0;

			/*
			 * ditto here for pselect(3).  testing showed that
			 * there are systems that do not get here.
			 */
#if defined HAVE_PSELECT && 0
			WAIT_PID();
#endif				/* HAVE_PSELECT && 0 */
		    }
		}

#if defined HAVE_PSELECT && 0
#else
		WAIT_PID();
#endif				/* HAVE_PSELECT && 0 */
	    }
	} while (0);

#if defined HAVE_PSELECT
	if (delete) {
	    sigaddset(&setv, SIGCHLD);
	}

	if (sigprocmask(SIG_SETMASK, &setv, NULL)) {
	    if (status) {
	    } else {
		status =
		    _libx1f4i0_lxport_stat_call(lxport, "sigprocmask", 11);
	    }
	}
    }
#endif				/* HAVE_PSELECT */

    return status;
}
