/*
 * lxrail-s.n.c
 * Copyright (C) 2011-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <fd.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxrail-inter.h>
#include <lxrail-types.h>

#define POST_BITS \
    X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA | X1f4_LXCALL_POST_LINE

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define lxrail(list) \
    ((struct lxrail_type *) (list))

typedef struct slip_type {
    int (*call)(void *, int, void *, struct x1f4_dispatch_type **), jack,
	miss, over, *reach;
    struct fxdata_type finite[1];
    struct fydata_type *fydata_data;
    struct lxrail_type *lxrail_data;
    struct x1f4_dispatch_type *dispatch_data;
    struct x1f4_dxcast_type dxcast, serial[1];
    struct x1f4_function_type function;
    struct x1f4_linetext_type linetext, *linetext_data;
    unsigned class;
    void *back, *clear[3], **order, **track;
} slip_type;

static int part_call(void *, int, void *, struct x1f4_dispatch_type **);

static int
part_call(void *back, int status, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    do {
	int slip;
	struct slip_type *call_data;

	if (status) {
	    break;
	}

	call_data = back;

	status = call_data->call(call_data->back, 0, output, dispatch);
	if (status) {
	    break;
	}

	slip = call_data->miss;
	slip--;
	if (slip) {
	    struct fydata_type *fydata_data;
	    struct lxrail_type *lxrail_data;
	    struct x1f4_dispatch_type *dispatch_data;
	    void **frame;

	    call_data->miss = slip;

	    lxrail_data = call_data->lxrail_data;

	    frame = (void *) (call_data + 1);
	    frame++;

	    fydata_data = call_data->fydata_data;

	    fydata_data++;

	    *call_data->reach = call_data->over;
	    *call_data->order = fydata_data;

	    call_data->fydata_data = fydata_data;

	    /* the args, count, flags set should do */
	    call_data->linetext = *call_data->linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    call_data->clear[2] = call_data->dispatch_data;

	    status = _libx1f4i0_lxcall_into_call
		(&call_data->linetext, &call_data->function, 0,
		 call_data->clear, POST_BITS | X1f4_LXCALL_POST_PLOT,
		 &lxrail_data->link_p, &lxrail_data->link_q,
		 &lxrail_data->link_e, lxrail_data->trap_e.eelookup_data, 1,
		 &call_data->jack, call_data->finite, call_data->serial);
	    if (status) {
		break;
	    }

	    dispatch_data = call_data->clear[2];

	    if (dispatch_data) {
		call_data->back = dispatch_data->back;
		call_data->call = dispatch_data->call;

		dispatch_data->back = call_data;
		dispatch_data->call = part_call;

		call_data->linetext.function.type = call_data->dxcast.type;

		*dispatch = dispatch_data;
	    }
	} else {
	    *dispatch = NULL;
	}
    } while (0);

    return status;
}


/*
 * function does setup for an unused output
 */
int
_libx1f4i0_lxrail_draw_call(void *lxrail, void *output, void **input,
			    struct x1f4_linetext_type *linetext_data,
			    unsigned count,
			    const struct x1f4_function_type *function_text,
			    int lead, int jack, int over, void *fydata)
{
    int linked, status;
    unsigned class, grade, parts, scale = 0;
    void *beta, **track;

    class = function_text->count - lead;

    grade = class + 1;

    parts = function_text->flags;

    if (parts & X1f4_E4_POST_TYPE) {
	if (linetext_data->function.flags
	    & (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
	    scale = 1;
	} else {
	    parts &= ~X1f4_E4_POST_TYPE;
	}
    }

    linked = sizeof(struct slip_type);

    status = lxrail(lxrail)->link_v.link
	(lxrail(lxrail)->link_v.data, &beta,
	 linked + (sizeof(void *) + (sizeof(int) << scale)) * grade
	 + sizeof(void *));
    if (status) {
	status = _libx1f4i0_lxrail_stat_link(lxrail);
    } else {
	int *args, type;
	struct fydata_type *fydata_data;
	struct slip_type *call_data;
	struct x1f4_function_type *function_call;
	void **clear, **frame, *series;

	call_data = beta;

	function_call = &call_data->function;

	function_call->count = grade;

	function_call->flags = parts & X1f4_E4_POST_TYPE;

	frame = (void *) ((char *) beta + linked);
	frame++;

	function_call->args = args = (void *) (frame + grade);

	track = input[1];

	frame[-1] = track[-1];

	type = linetext_data->function.type;

	series = input[0];

	if (type != X1f4_E4_CASE) {
	    l_USER(&call_data->dxcast, series);

	    call_data->dxcast.call = 0;
	    call_data->dxcast.type = type;

	    series = &call_data->dxcast.data;
	} else {
	}

	_libx1f4i0_lxcall_hold_call
	    (frame, track, parts, args, function_text->args, class, lead,
	     jack);

	args[jack] = over;

	call_data->order = frame + jack;

	call_data->reach = args + jack;

	fydata_data = fydata;

	*call_data->order = fydata_data;

	clear = call_data->clear;

	clear[0] = series;
	clear[1] = frame;

	call_data->linetext = *linetext_data;

	call_data->linetext.function.type = X1f4_E4_CASE;

	status = _libx1f4i0_lxcall_post_call
	    (&call_data->linetext, function_call, output, 0, clear, POST_BITS,
	     &lxrail(lxrail)->link_v, &lxrail(lxrail)->link_p,
	     &lxrail(lxrail)->link_q, &lxrail(lxrail)->link_e,
	     lxrail(lxrail)->trap_e.eelookup_data);
	if (status) {
	} else {
	    struct x1f4_dispatch_type *dispatch_data;

	    dispatch_data = input[2] = clear[2];

	    if (dispatch_data) {
		call_data->back = dispatch_data->back;
		call_data->call = dispatch_data->call;
		call_data->miss = count;
		call_data->over = over;

		call_data->class = class;
		call_data->track = track;

		call_data->jack = jack;

		call_data->fydata_data = fydata_data;
		call_data->lxrail_data = lxrail;

		call_data->linetext_data = linetext_data;

		dispatch_data->back = call_data;
		dispatch_data->call = part_call;

		call_data->linetext.function.type = type;

		call_data->dispatch_data = dispatch_data;
	    }
	}
    }

    return status;
}
