/*
 * lxroll-l.a.c
 * Copyright (C) 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <fcntl.h>
#include <unistd.h>

#include <arpa/inet.h>

#include <netinet/in.h>
#include <netinet/tcp.h>

#include <sys/socket.h>
#include <sys/types.h>

#include <e4.h>
#include <lxfile.h>
#include <lxroll-defs.h>
#include <lxroll-inter.h>
#include <lxroll-types.h>

#define I_FILE(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))

int
_libx1f4i0_lxroll_a_logique(void *context, void *output, void **input)
{
    int fd = -1, ld = -1, status;

    do {
	int server;
	socklen_t size;
        struct lxroll_type *lxroll_data;
	struct sockaddr address;
        struct x1f4_dxfile_type *dxfile_data, *dxfile_slip, *dxfile_text;
	unsigned bits;

        lxroll_data = context;

        dxfile_slip = I_FILE(input[2]);

        if (dxfile_slip->file) {
	    status = x1f4_bill_dxfile(dxfile_slip, &server);
	    if (status) {
		status = _libx1f4i0_lxroll_stat_side(lxroll_data);
		if (1) {
		    break;
		}
	    } else {
	    }
	} else {
	    status = _libx1f4i0_lxroll_stat_flat(lxroll_data);
            if (1) {
                break;
            }
	}

        dxfile_data = I_FILE(input[0]);

        dxfile_text = I_FILE(input[1]);

        if (dxfile_data == dxfile_text) {
	    status = _libx1f4i0_lxroll_stat_back(lxroll_data);
            if (1) {
                break;
            }
	}

	/*
	 * accept(2) before closing any files.
	 */

	size = sizeof(address);
	fd = accept(server, (struct sockaddr *) &address, &size);
	if (fd == -1) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll_data, "accept");
	    if (1) {
		break;
	    }
	}

        bits = I_MODE(input[3]);

	if (bits & TCP_NO_DELAY) {
	    int part = 1;

	    status = setsockopt
		(fd, IPPROTO_TCP, TCP_NODELAY, &part, sizeof(int));
	    if (status) {
		status = _libx1f4i0_lxroll_stat_part
		    (lxroll_data, TCP_NO_DELAY);
		if (1) {
		    break;
		}
	    }
	}

	ld = dup(fd);
	if (ld == -1) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll_data, "dup");
	    if (1) {
		break;
	    }
	}

	status = _libx1f4i0_lxroll_land_roll
	    (lxroll_data, bits, dxfile_data, &fd, dxfile_text, &ld);
    } while (0);

    if (fd ^ -1) {
	close(fd);
    }

    if (ld ^ -1) {
	close(ld);
    }

    return status;
}
