/*
 * lxtrap-i.d.c
 * Copyright (C) 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <ln.h>
#include <lxcast.h>
#include <lxtrap-close.h>
#include <lxtrap-drive.h>
#include <lxtrap-types.h>

int
_libx1f4i0_lxtrap_node_args(struct dxnews_type *line_data, int *args,
			    unsigned fold,
			    struct x1f4_nodelink_type *nodelink_data)
{
    int status = 0;
    struct lxtrap_type *lxtrap_data;
    unsigned i;
    void **track;

    lxtrap_data = line_data->lxtrap_data;

    track = line_data->track;

    i = fold;

    do {
	int type;
	const struct lxtype_type *lxtype_data;
	unsigned miss;
	void *side;

	side = *track;

	type = *args++;

	if (type == X1f4_E4_CASE) {
	    struct x1f4_dxcast_type *dxcast_data;

	    dxcast_data = I_USER(side);
	    type = dxcast_data->type;
	    side = &dxcast_data->data;
	}

	lxtype_data = lxtrap_data->link_t.data;

	close_frame(miss, lxtype_data, lxtrap_data);

	if (miss) {
	    if (lxtype_data->bits & X1f4_LX_LINK_ACCESS) {
		status = lxtype_data->node
		    (lxtype_data->text, side, I_USER(side), nodelink_data);
		if (status) {
		    break;
		}
	    } else {
		int (*copy)(void *, void *, void **, const void *,
			    const struct x1f4_nodelink_type *);

		copy = lxtype_data->copy;
		if (copy) {
		    status = copy
			(lxtype_data->text, NULL, side, I_USER(side),
			 nodelink_data);
		    if (status) {
			break;
		    }
		}
	    }
	} else {
	}

	i--;

	track++;
    } while (i);

    line_data->fold = fold - i;

    return status;
}
