/*
 * lxtrap-i.t.c
 * Copyright (C) 2011-2015, 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <exerrors.h>
#include <fd.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>
#include <retain.h>

#if __RETAIN_OBJECT_DATA__
# error data for created objects is not retained, as probably should have been
#endif				/* __RETAIN_OBJECT_DATA__ */

#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define l_MODE(e, output) \
    {									      \
	X1f4_E4_C_MODE *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

extern const void *const x1f4_a1_walk_link;

static int pick_call(void *, void **, void *, void **, void **,
		     struct lxtrap_type *, struct x1f4_linetext_type *, int);

static int
pick_call(void *output, void **input, void *meta, void **track, void **side,
	  struct lxtrap_type *lxtrap_data,
	  struct x1f4_linetext_type *linetext_data, int frame)
{
    int status;
    struct fxdata_type missed;
    void *zeta;

    zeta = track[-1];

    track[-1] = meta;

    status = _libx1f4i0_lxtrap_pick_call
	(&missed, 1, input, track, side, lxtrap_data, linetext_data, frame);
    if (status) {
	if (status == X1f4_EX_CAN_CONTINUE) {
	    status = 0;
	    l_MODE(-1, output);
	} else {
	}
    } else {
	l_MODE(0, output);
    }

    track[-1] = zeta;

    return status;
}


int
_libx1f4i0_lxtrap_trap_call(void *context, void *output, void **input,
			    int frame)
{
    int linked, shift, status;
    struct lxtrap_type *lxtrap_data;
    struct x1f4_dxcast_type *dxcast_data;
    void **track;

    linked = output == x1f4_a1_walk_link;

    lxtrap_data = context;

    if (linked) {
	track = input[1];
    } else {
	track = input;
    }

    shift = frame ^ 2 ? 0 : 1;

    dxcast_data = I_USER(track[shift]);

    if (X1f4_E4_CALL < dxcast_data->type) {
	struct x1f4_linetext_type *linetext_data;

	linetext_data = I_USER(&dxcast_data->data);
	if (linetext_data->function.function) {
	    void *function, *meta;

	    meta = *(track - 1);

	    shift++;

	    track += shift;

	    function = ((void **) meta)[X1f4_E4_RENDITION];

	    status = _libx1f4i0_lxcall_even_list
		(&linetext_data->function, function, shift, &track, 0,
		 &lxtrap_data->link_v, &lxtrap_data->link_p,
		 &lxtrap_data->link_q, &lxtrap_data->link_e,
		 lxtrap_data->trap_e.eelookup_data);
	    if (status) {
	    } else {
		if (linked) {
		    status = _libx1f4i0_lxtrap_slip_call
			(output, 0, NULL, 1, input, meta, track, function,
			 lxtrap_data, linetext_data, frame);
		} else {
		    status = pick_call
			(output, input, meta, track, function, lxtrap_data,
			 linetext_data, frame);
		}
	    }
	} else {
	    status = _libx1f4i0_lxtrap_stat_zero(lxtrap_data);
	}
    } else {
	status = _libx1f4i0_lxtrap_stat_call(lxtrap_data, shift);
    }

    return status;
}
