/*
 * lxtrap-i.u.c
 * Copyright (C) 2011-2015, 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e4.h>
#include <exerrors.h>
#include <fd.h>
#include <lxcall.h>
#include <lxtrap-types.h>
#include <track.h>

#define l_TEXT(e, output) \
    {									      \
	X1f4_E4_C_TEXT *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

extern const char *const x1f4_c1_empty_string;
extern const struct x1f4_track_type _libx1f4i0_missed_track,
    _libx1f4i0_stored_track;

int
_libx1f4i0_lxtrap_pick_call(void *output, int offset, void **input,
			    void **track, void *side,
			    struct lxtrap_type *lxtrap_data,
			    struct x1f4_linetext_type *linetext_data,
			    int frame)
{
    int status;

    do {
	void *deftext;

	if (linetext_data->function.flags
	    & (X1f4_E4_SIDE_LIST | X1f4_E4_SLIP_LIST)) {
	    if (frame == 2) {
		offset++;
	    }

	    status = _libx1f4i0_lxcall_pick_call
		(input, side, offset, 0, &lxtrap_data->link_v,
		 &lxtrap_data->link_e, &linetext_data);
	    if (status) {
		break;
	    }
	} else {
	    ((void **) track[-1])[X1f4_E4_RENDITION] =
		&linetext_data->function;
	}

	deftext = (void *) linetext_data->context;

	if (frame) {
	    struct x1f4_track_type **trace, *track_data;

	    trace = lxtrap_data->link_h.call;

	    track_data = *trace;

	    if (frame == 1) {
		*trace = (struct x1f4_track_type *) &_libx1f4i0_missed_track;

		status = linetext_data->function.function
		    (deftext, output, track);
	    } else {
		struct fxcase_type misill;
		struct x1f4_track_type reset;

		reset = _libx1f4i0_stored_track;

		reset.data = &misill;

		*trace = &reset;

		misill.missed = 1;
		misill.text = (X1f4_E4_C_TEXT) x1f4_c1_empty_string;
		misill.trans = lxtrap_data->link_m;

		status = linetext_data->function.function
		    (deftext, output, track);
		if (status) {
		    do {
			if (misill.missed) {
			    break;
			}

			if (status == X1f4_EX_CAN_CONTINUE) {
			    int excess;
			    void *store;

			    store = *input;

			    excess = _libx1f4i0_lxcall_pick_text
				(&lxtrap_data->ever_a, store);
			    if (excess) {
				status = excess;
			    } else {
				l_TEXT(misill.text, store);
				if (1) {
				    break;
				}
			    }
			}

			if (misill.text != x1f4_c1_empty_string) {
			    lxtrap_data->link_m.free
				(lxtrap_data->link_m.data, misill.text);
			}
		    } while (0);
		}
	    }

	    *trace = track_data;
	} else {
	    status = linetext_data->function.function(deftext, output, track);
	}
    } while (0);

    return status;
}
