/*
 * lxvery-a.e.c
 * Copyright (C) 2008-2015, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxvery-config.h>

#include <d_fset.h>
#include <e4.h>
#include <er.h>
#include <exerrors.h>
#include <lxcall.h>
#include <lxvery-inter.h>
#include <lxvery-types.h>

#define lxvery(lxvery) \
    ((struct lxvery_type *) (lxvery))

int
_libx1f4i0_lxvery_stat_ever(void *lxvery)
{
    LINE_EEER(lxvery(lxvery)->link_e);

    PUSH_DATA("key exists", 10);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxvery_stat_line(void *lxvery, int shadow, int error)
{
    int status;

    if (error == X1f4_BQFSET_DECK_ERROR) {
	status = _libx1f4i0_lxvery_stat_null(lxvery);
    } else {
	if (error == X1f4_BQFSET_EVER_MATCH) {
	    status = _libx1f4i0_lxvery_stat_ever(lxvery);
	} else {
	    if (error == X1f4_BQFSET_FREE_ERROR) {
		status = _libx1f4i0_lxvery_stat_free(lxvery, 0);
	    } else {
		if (error == X1f4_BQFSET_LINK_ERROR) {
		    status = _libx1f4i0_lxvery_stat_link(lxvery);
		} else {
		    if (error == X1f4_BQFSET_MODE_ERROR) {
			status = _libx1f4i0_lxvery_stat_mode(lxvery);
		    } else {
			/*
			 * there is defined a _TEXT_ERROR condition, but the
			 * condition is never met when writing this.
			 */
			status = X1f4_EX_CRITICAL;
		    }
		}
	    }
	}
    }

    return _libx1f4i0_lxcall_land_slip(status, shadow);
}


int
_libx1f4i0_lxvery_stat_lock(void *lxvery, int mode)
{
    LINE_EEER(lxvery(lxvery)->link_e);

    PUSH_DATA("non intrinsic integer ", 22);
    PUSH_MODE(mode + 1);
    PUSH_RANK(mode);
    PUSH_DATA(" index argument", 15);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxvery_stat_miss(void *lxvery, long text)
{
    LINE_EEER(lxvery(lxvery)->link_e);

#if __VERY_PSET__
    text -= ~(X1f4_E4_C_BILL) 0 >> 1;
#endif				/* __VERY_PSET__ */

    PUSH_DATA("no mapping for key `", 20);
    PUSH_LONG(text);
    PUSH_DATA("'", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxvery_stat_null(void *lxvery)
{
    LINE_EEER(lxvery(lxvery)->link_e);

    PUSH_DATA("void set", 8);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxvery_stat_post(void *lxvery, long text)
{
    LINE_EEER(lxvery(lxvery)->link_e);

#if __VERY_PSET__
    text -= ~(X1f4_E4_C_BILL) 0 >> 1;
#endif				/* __VERY_PSET__ */

    PUSH_DATA("key `", 5);
    PUSH_LONG(text);
    PUSH_DATA("' exists", 8);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxvery_stat_text(void *lxvery, int error, long text)
{
    int status;

    if (error == X1f4_BQFSET_EVER_MATCH) {
	status = _libx1f4i0_lxvery_stat_post(lxvery, text);
    } else {
	if (1) {
	    status = _libx1f4i0_lxvery_stat_line(lxvery, 0, error);
	}
    }

    return status;
}
