/*
 * lxvery-c.f.c
 * Copyright (C) 2013-2019, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <exerrors.h>
#include <lxcast.h>
#include <lxvery-drive.h>
#include <lxvery-inter.h>

static int line_trap(void *, void *, void **);

static int
line_trap(void *context, void *output, void **input)
{
    int status;
    struct lxvery_type *lxvery_data;
    struct x1f4_function_type *function_data;
    unsigned count;

    lxvery_data = context;

    function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

    status = 0;

    count = function_data->count - 1;
    if (count) {
	struct x1f4_function_type function;
	void *slip[3], **strip;

	strip = input[-1];

	slip[0] = strip;
	slip[1] = *input;

	/*
	 * pretty error reporting:
	 */
	function.length = 5;
	function.name = "g_fix";

	function.args = function_data->args;

	strip[X1f4_E4_RENDITION] = &function;

	do {
	    slip[2] = *++input;
	    status = _libx1f4i0_lxvery_p_forward(lxvery_data, NULL, slip + 1);
	    if (status) {
		break;
	    }

	    function.args++;
	} while (count -= 1);
    }

    return status;
}


int
_libx1f4i0_lxvery_f_centric(void *context, void *output, void **input)
{
    int status;

    do {
	const int *args;
	struct lxvery_type *lxvery_data;
	struct x1f4_function_type *function_data;
	unsigned count;

	lxvery_data = context;

	function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

	count = function_data->count;

	args = function_data->args + 1;
	for (--count; count; count--) {
	    if (*args != X1f4_E4_MODE) {
		break;
	    } else {
		args++;
	    }
	}
	if (count) {
	    status = _libx1f4i0_lxvery_stat_lock
		(lxvery_data, function_data->count - count);
	    if (1) {
		break;
	    }
	}

	status = line_trap(context, output, input);
	if (status) {
	} else {
	    l_LINE(I_LINE(*input), output);
	}
    } while (0);

    return status;
}
