/*
 * lxvery-c.r.c
 * Copyright (C) 2008-2011, 2013-2019, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <exerrors.h>
#include <fd.h>
#include <lxvery-drive.h>
#include <lxvery-inter.h>
#include <lxvery-types.h>

static int line_trap(void *, void *, void **);
static int rule_line(struct lxvery_type *, void *, void **);
static int slip_line(void *news);

static int
line_trap(void *context, void *output, void **input)
{
    int status;
    struct lxvery_type *lxvery_data;
    void **mind, *news;

    lxvery_data = context;

    status = lxvery_data->link_v.pick
	(lxvery_data->link_v.data, &mind, slip_line, sizeof(void *) << 1,
	 &news);
    if (status) {
	status = X1f4_EX_CRITICAL;
    } else {
	void *line, **seek;

	*mind = news;

	seek = news;

	*seek++ = lxvery_data;

	status = _libx1f4i0_lxvery_line_line(lxvery_data, &line);
	if (status) {
	    *seek = NULL;
	} else {
	    *seek = line;

	    l_LINE(line, output);

	    status = rule_line(lxvery_data, line, input);
	}
    }

    return status;
}


static int
rule_line(struct lxvery_type *lxvery_data, void *line, void **input)
{
    int status = 0;
    struct x1f4_function_type *function_data;
    unsigned count;

    function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

    count = function_data->count;
    if (count) {
	struct fxdata_type fxdata;
	struct x1f4_function_type function;
	void *slip[3], **strip;

	strip = input[-1];

	slip[0] = strip;
	slip[1] = &line;

	/*
	 * pretty error reporting:
	 */
	function.length = 8;
	function.name = "g_render";

	function.args = function_data->args - 1;

	strip[X1f4_E4_RENDITION] = &function;

	do {
	    slip[2] = *input++;
	    status = _libx1f4i0_lxvery_p_forward
		(lxvery_data, &fxdata, slip + 1);
	    if (status) {
		break;
	    }

	    function.args++;
	} while (count -= 1);
    }

    return status;
}


static int
slip_line(void *news)
{
    int status;
    void *line, **side;

    side = news;
    line = side[1];
    if (line) {
	status = _libx1f4i0_lxvery_slip_line(*side, &line);
    } else {
	status = 0;
    }

    return status;
}


int
_libx1f4i0_lxvery_r_centric(void *context, void *output, void **input)
{
    int status;

    do {
	const int *args;
	struct lxvery_type *lxvery_data;
	struct x1f4_function_type *function_data;
	unsigned count;

	lxvery_data = context;

	function_data = ((void **) input[-1])[X1f4_E4_RENDITION];

	count = function_data->count;

	args = function_data->args;
	for (; count; count--) {
	    if (*args != X1f4_E4_MODE) {
		break;
	    } else {
		args++;
	    }
	}
	if (count) {
	    status = _libx1f4i0_lxvery_stat_lock
		(lxvery_data, function_data->count - count);
	    if (1) {
		break;
	    }
	}

	status = line_trap(context, output, input);
    } while (0);

    return status;
}
