/*
 * qsce.r.c
 * Copyright (C) 2013, 2014, 2016, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <qscc-config.h>

#include <stddef.h>
#include <string.h>

#include <qscc-defs.h>
#include <qscc-inter.h>
#include <qscc-names.h>
#include <qscc-types.h>

#define true(e)				1

#define qsrate(qsrate)			((struct qsrate_type *) (qsrate))

static int fine_rate(struct qsrate_type *, unsigned);
static int flat_rate(struct qsrate_type *, struct qsrate_type *, unsigned,
		     unsigned);
static int long_rate(void *, unsigned, unsigned, int);
static int pull_rate(struct qsrate_type *, unsigned);
static int tear_rate(struct qsrate_type *, struct qsrate_type *, unsigned,
		     unsigned);

static int
fine_rate(struct qsrate_type *qsrate_data, unsigned rate)
{
    int status;

    do {
	struct fplink_type *fplink_data;
	unsigned miss = 0;

	if (rate) {
	} else {
	    status = 0;
	    if (1) {
		break;
	    }
	}

	if (qsrate_data->link_e.fplink_text) {
	} else {
	    status = LINK_ERROR;
	    if (1) {
		break;
	    }
	}

	rate--;

	fplink_data = qsrate_data->link_e.fplink_data;
	while (fplink_data) {
	    miss++;
	    fplink_data = fplink_data->fplink_data;
	}

	if (miss ^ rate) {
	    status = LINK_ERROR;
	    if (1) {
		break;
	    }
	}

	status = 0;
    } while (0);

    return status;
}


static int
flat_rate(struct qsrate_type *qsrate_data, struct qsrate_type *qsrate_text,
	  unsigned size, unsigned tear)
{
    int status;

    if (size) {
	unsigned mall, part;
	void *data, *text;

	part = size - tear;

	mall = qsrate_data->link_a.fplist.mall;

	data = qsrate_data->link_a.fplist.fpnode;

	text = qsrate_text->link_a.fplist.fpnode;

#if __MICRO_RATES__
	do {
	    unsigned line, park, toll;

	    line = qsrate_data->link_a.fplist.line;
	    toll = 3 * line;
	    while (~line & 2) {
		line >>= 1;
	    }
	    line *= 3;
	    line >>= 1;
	    if (part < line) {
		toll = line;
	    } else {
		while (!(toll < part) && ~toll & 1) {
		    toll >>= 1;
		}
		toll <<= 1;
	    }

	    if (line < toll) {
		status = qsrate_text->link_m.mode
		    (qsrate_text->link_m.data, &text, mall * toll);
		if (status) {
		    status = MODE_ERROR;
		    if (1) {
			break;
		    }
		} else {
		    qsrate_text->link_a.fplist.fpnode = text;
		}
	    }
#endif				/* __MICRO_RATES__ */

	    memcpy(text, (char *) data + mall * tear, mall * part);

	    qsrate_data->link_a.fplist.size = tear;
	    qsrate_text->link_a.fplist.size = part;

#if __MICRO_RATES__
	    toll = qsrate_data->link_a.fplist.line * 3;
	    park = toll;
	    if (tear < line) {
		toll = line;
	    } else {
		while (!(toll < tear) && ~toll & 1) {
		    toll >>= 1;
		}
		toll <<= 1;
	    }
	    if (size < line) {
		park = line;
	    } else {
		while (!(park < size) && ~park & 1) {
		    park >>= 1;
		}
		park <<= 1;
	    }

	    if (toll < park) {
		status = qsrate_data->link_m.mode
		    (qsrate_data->link_m.data, &data, mall * toll);
		if (status) {
		    status = MODE_ERROR;
		    if (1) {
			break;
		    }
		} else {
		    qsrate_data->link_a.fplist.fpnode = data;
		}
	    }
#endif				/* __MICRO_RATES__ */

	    status = 0;
#if __MICRO_RATES__
	} while (0);
#endif				/* __MICRO_RATES__ */
    } else {
	status = 0;
    }

    return status;
}


static int
long_rate(void *qsrate, unsigned head, unsigned tail, int hash)
{
    int able, status;
    unsigned count, lock, part, rate;

    count = qsrate(qsrate)->link_a.fplist.size;

    rate = qsrate(qsrate)->link_a.fplist.rate;

    tail++;

    lock = qsrate(qsrate)->link_a.fplist.lock;

    status = _libx1f4l2_mile_qsrate
	(qsrate, lock, rate, 0, head, tail, count,
	 &qsrate(qsrate)->link_a.fplist.fpnode,
	 &qsrate(qsrate)->link_a.fplist.node, &able, &part, hash);
    if (status) {
    } else {
	qsrate(qsrate)->link_a.fplist.size = count - tail + head;
	if (rate) {
	    if (qsrate(qsrate)->link_a.fplist.node < 2) {
		if (true(qsrate(qsrate)->link_a.fplist.node)) {
		    status = _libx1f4l2_wrap_qsrate(qsrate, lock, rate - 1);
		} else {
		}
	    }
	}
    }

    return status;
}


static int
pull_rate(struct qsrate_type *qsrate_data, unsigned rate)
{
    int status;
    void *zero;

    zero = qsrate_data->link_a.fplist.fpnode;

    status = _libx1f4l2_mall_qsrate(qsrate_data, rate);
    if (status) {
    } else {
#if __MICRO_RATES__
	do {
	    status = qsrate_data->link_m.mode
		(qsrate_data->link_m.data, &zero,
		 qsrate_data->link_a.fplist.line * 3
		 * qsrate_data->link_a.fplist.mall >> 1);
	    if (status) {
		status = MODE_ERROR;
	    }
#endif				/* __MICRO_RATES__ */

	    qsrate_data->link_e.fplink_text = zero;
#if __MICRO_RATES__
	} while (0);
#endif				/* __MICRO_RATES__ */
    }

    return status;
}


static int
tear_rate(struct qsrate_type *qsrate_data, struct qsrate_type *qsrate_text,
	  unsigned size, unsigned tear)
{
    int status;

    qsrate_text->link_a.fplist.node = qsrate_data->link_a.fplist.node;
    qsrate_text->link_a.fplist.rate = qsrate_data->link_a.fplist.rate;
    qsrate_text->link_a.fplist.size = qsrate_data->link_a.fplist.size;

    qsrate_text->link_a.fplist.fpnode = qsrate_data->link_a.fplist.fpnode;

    status = long_rate(qsrate_text, 0, tear - 1, 1);
    if (status) {
    } else {
	status = long_rate(qsrate_data, tear, size - 1, 0);
	if (status) {
	} else {
	    if (status) {
	    } else {
		status =
		    fine_rate(qsrate_text, qsrate_text->link_a.fplist.rate);
	    }
	}
    }

    return status;
}


int
x1f4_tear_qsrate(void *qsdata, unsigned tear, void *qstext)
{
    int status;

    if (qsrate(qstext)->link_a.fplist.size) {
	status = SIZE_ERROR;
    } else {
	unsigned size;

	size = qsrate(qsdata)->link_a.fplist.size;
	if (size < tear) {
	    status = DECK_ERROR;
	} else {
	    if (tear) {
		unsigned rate;

		rate = qsrate(qsdata)->link_a.fplist.rate;
		if (rate) {
		    if (tear < size) {
			status = pull_rate(qstext, rate);
			if (status) {
			} else {
			    status = tear_rate(qsdata, qstext, size, tear);
			}
		    } else {
			if (1) {
			    status = 0;
			} else {
			}
		    }
		} else {
		    status = flat_rate(qsdata, qstext, size, tear);
		}
	    } else {
		status = _libx1f4l2_swap_qsrate(qsdata, qstext);
	    }
	}
    }

    return status;
}
