/*
 * real.e.c
 * Copyright (C) 2006-2012, 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <float.h>
#include <math.h>

#if defined LIBx1f4i0
# include <ctype.0.h>
#endif				/* LIBx1f4i0 */

#if !defined LIBx1f4i0
# define C_TYPE_X(c_type_x, c) \
    ((c_type_x)[(c) >> 5] & 1 << ((c) & 31))
#endif				/* !LIBx1f4i0 */

#if !defined DBL_MANT_DIG
# if SIZEOF_DOUBLE == 8
#  undef FLT_RADIX
#  define FLT_RADIX			2
#  define DBL_MANT_DIG			53
# endif				/* SIZEOF_DOUBLE == 8 */
#endif				/* DBL_MANT_DIG */

#if defined DBL_MANT_DIG
# if defined FLT_RADIX
#  if FLT_RADIX == 2
#   define __USE_DOUBLE_MANTISSA_DIGITS__
#  endif			/* FLT_RADIX == 2 */
# endif				/* FLT_RADIX */
#endif				/* DBL_MANT_DIG */

#undef Digits

#if defined __USE_DOUBLE_MANTISSA_DIGITS__
# if DBL_MANT_DIG == 53
#  if FLT_RADIX == 2
#   define Digits			15
#  endif			/* FLT_RADIX == 2 */
# endif				/* DBL_MANT_DIG == 53 */
#endif				/* __USE_DOUBLE_MANTISSA_DIGITS__ */

static const double pow2s[] = {
/* *INDENT-OFF* */
    1e-1,
    1e-2,
    1e-4,
    1e-8,
    1e-16,
    1e-32,
    1e-64,
    1e-128,
    1e-256,
/* *INDENT-ON* */
};
static const unsigned c_type_1[] = {
/*
 *                   . .     .
 *
 * .                     +
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 */
/* *INDENT-OFF* */
    0x00002600, 0x00000801, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
/* *INDENT-ON* */
}, c_type_2[] = {
/*
 *
 *
 *
 * 0 1 2 3 4 5 6 7 8 9
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *
 */
/* *INDENT-OFF* */
    0x00000000, 0x03ff0000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
/* *INDENT-ON* */
};

int
x1f4_parse_ereal(double *real, const char *s, const char **e)
{
    const char *t;
    int c, digital, sign = 0, status = 0;

    t = s;
    c = *(const unsigned char *) t;
    do {
	if (c == '-') {
	    sign ^= 1;
	} else if (C_TYPE_X(c_type_1, c)) {
	} else {
	    break;
	}
	t++;
	c = *(const unsigned char *) t;
    } while (1);

    digital = C_TYPE_X(c_type_2, c);
    if (c != '.' && !digital) {
	status = 1;
    } else {
	do {
	    double integral = 0, unit = 0;
	    int d = 0, exponent = 0;
#if defined __USE_DOUBLE_MANTISSA_DIGITS__
	    unsigned D;
#else
# define D				64
#endif				/* __USE_DOUBLE_MANTISSA_DIGITS__ */

	    while (c == '0') {
		t++;
		c = *(const unsigned char *) t;
	    }
	    if (C_TYPE_X(c_type_2, c)) {
		do {
		    integral *= 10;
		    integral += c - '0';
		    t++;
		    c = *(const unsigned char *) t;
		} while (C_TYPE_X(c_type_2, c));
	    }
	    if (c != '.') {
		if (c) {
		    status = 1;
		}

		s = t;
	    } else {
		t++;
		c = *(const unsigned char *) t;
		if (!C_TYPE_X(c_type_2, c)) {
		    status = 1;

		    if (digital) {
			s = t - 1;
		    } else {
			break;
		    }
		} else {
		    const char *u, *v;
		    unsigned e, o;

#if defined Digits
		    D = Digits;
#elif defined __USE_DOUBLE_MANTISSA_DIGITS__
		    D = floor(DBL_MANT_DIG * log(2) / log(10));
#endif				/* __USE_DOUBLE_MANTISSA_DIGITS__ */

		    s = t;

#if defined __USE_DOUBLE_MANTISSA_DIGITS__
		    D += 2;
#endif				/* __USE_DOUBLE_MANTISSA_DIGITS__ */

		    while (c == '0') {
			d++;
			t++;
			c = *(const unsigned char *) t;
		    }
		    if (integral) {
			e = d;
			d = 0;
		    } else {
			e = 0;
		    }
		    u = t;
		    o = e;
		    while (C_TYPE_X(c_type_2, c) && o < D) {
			o++;
			t++;
			c = *(const unsigned char *) t;
		    }
		    if (t != u) {
			v = t - 1;
			while (*v == '0') {
			    v--;
			}
			v++;
			e += v - u;
#if SIZEOF_UNSIGNED_LONG != 8
			do {
			    unit *= 10;
			    unit += *u++ - '0';
			} while (u < v);
#else
			if (1) {
			    unsigned long used = 0;

			    do {
				used = ((used << 2) + used) << 1;
				used += *u++ - '0';
			    } while (u < v);
			    unit = used;
			}
#endif				/* SIZEOF_UNSIGNED_LONG != 8 */
		    }
		    if (o == D) {
			while (C_TYPE_X(c_type_2, c)) {
			    t++;
			    c = *(const unsigned char *) t;
			}
		    }
		    if (c) {
			status = 1;
		    }

		    s = t;

		    d += e;
		}
	    }

	    if (c == 'e') {
		int side;

		t = s;

		t++;
		c = *(const unsigned char *) t;
		if (c == '-') {
		    side = 1;
		    t++;
		    c = *(const unsigned char *) t;
		} else {
		    side = 0;
		    if (c == '+') {
			t++;
			c = *(const unsigned char *) t;
		    }
		}

		if (C_TYPE_X(c_type_2, c)) {
		    do {
			exponent *= 10;
			exponent += c - '0';
			t++;
			c = *(const unsigned char *) t;
		    } while (C_TYPE_X(c_type_2, c));

		    s = t;

		    if (side) {
			exponent = -exponent;
		    }

		    integral *= pow(10, exponent);

		    if (c) {
		    } else {
			status = 0;
		    }
		}
	    }

	    if (unit) {
		if (d < exponent) {
		    unit *= pow(10, exponent - d);
		} else {
		    d -= exponent;

		    if (511 < d) {
			unit = 0;
		    } else {
#if defined __USE_DOUBLE_MANTISSA_DIGITS__
			D -= 2;
#endif				/* __USE_DOUBLE_MANTISSA_DIGITS__ */

#if defined __USE_DOUBLE_MANTISSA_DIGITS__
			if (d < D + 1 || 1) {
			    unit /= pow(10, d);
			} else {
#elif 0
			}
#else
			if (0) {
			} else {
#endif				/* __USE_DOUBLE_MANTISSA_DIGITS__ */
			    unsigned f;

			    /*
			     * starting with the higher powers seems to produce
			     * a more accurate result.
			     */
			    f = 9;
			    while (f) {
				f--;
				if (d & 1 << f) {
				    unit *= pow2s[f];
				}
			    }
			}
		    }
		}
	    }

	    integral += unit;
	    if (sign) {
		*real = -integral;
	    } else {
		*real = integral;
	    }
	} while (0);
    }

    if (e) {
	*e = s;
    }

    return status;
}
